@@ -16,15 +16,14 @@
 
 """Heat exception subclasses"""
 
-import functools
 import sys
 
+from oslo_log import log as logging
 import six
 from six.moves.urllib import parse as urlparse
 
-from heat.openstack.common.gettextutils import _
-from heat.openstack.common import log as logging
-
+from heat.common.i18n import _
+from heat.common.i18n import _LE
 
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
@@ -79,7 +78,7 @@ def wrap_exception(notifier=None, publisher_id=None, event_type=None,
                     temp_type = event_type
                     if not temp_type:
                         # If f has multiple decorators, they must use
-                        # functools.wraps to ensure the name is
+                        # six.wraps to ensure the name is
                         # propagated.
                         temp_type = f.__name__
 
@@ -89,7 +88,7 @@ def wrap_exception(notifier=None, publisher_id=None, event_type=None,
                 # re-raise original exception since it may have been clobbered
                 raise exc_info[0], exc_info[1], exc_info[2]
 
-        return functools.wraps(f)(wrapped)
+        return six.wraps(f)(wrapped)
     return inner
 
 
@@ -110,9 +109,9 @@ class HeatException(Exception):
             self.message = self.msg_fmt % kwargs
         except KeyError:
             exc_info = sys.exc_info()
-            #kwargs doesn't match a variable in the message
-            #log the issue and the kwargs
-            LOG.exception(_('Exception in string format operation'))
+            # kwargs doesn't match a variable in the message
+            # log the issue and the kwargs
+            LOG.exception(_LE('Exception in string format operation'))
             for name, value in six.iteritems(kwargs):
                 LOG.error("%s: %s" % (name, value))  # noqa
 
@@ -134,8 +133,8 @@ class MissingCredentialError(HeatException):
 
 
 class BadAuthStrategy(HeatException):
-    msg_fmt = _("Incorrect auth strategy, expected \"%(expected)s\" but "
-                "received \"%(received)s\"")
+    msg_fmt = _('Incorrect auth strategy, expected "%(expected)s" but '
+                'received "%(received)s"')
 
 
 class AuthBadRequest(HeatException):
@@ -158,7 +157,7 @@ class Forbidden(HeatException):
     msg_fmt = _("You are not authorized to complete this action.")
 
 
-#NOTE(bcwaldon): here for backwards-compatibility, need to deprecate.
+# NOTE(bcwaldon): here for backwards-compatibility, need to deprecate.
 class NotAuthorized(Forbidden):
     msg_fmt = _("You are not authorized to complete this action.")
 
@@ -215,8 +214,8 @@ class InvalidTemplateAttribute(HeatException):
 
 
 class InvalidTemplateReference(HeatException):
-    msg_fmt = _("The specified reference \"%(resource)s\" (in %(key)s)"
-                " is incorrect.")
+    msg_fmt = _('The specified reference "%(resource)s" (in %(key)s)'
+                ' is incorrect.')
 
 
 class UserKeyPairMissing(HeatException):
@@ -231,6 +230,26 @@ class ImageNotFound(HeatException):
     msg_fmt = _("The Image (%(image_name)s) could not be found.")
 
 
+class ServerNotFound(HeatException):
+    msg_fmt = _("The server (%(server)s) could not be found.")
+
+
+class VolumeNotFound(HeatException):
+    msg_fmt = _("The Volume (%(volume)s) could not be found.")
+
+
+class VolumeSnapshotNotFound(HeatException):
+    msg_fmt = _("The VolumeSnapshot (%(snapshot)s) could not be found.")
+
+
+class VolumeTypeNotFound(HeatException):
+    msg_fmt = _("The VolumeType (%(volume_type)s) could not be found.")
+
+
+class NovaNetworkNotFound(HeatException):
+    msg_fmt = _("The Nova network (%(network)s) could not be found.")
+
+
 class PhysicalResourceNameAmbiguity(HeatException):
     msg_fmt = _(
         "Multiple physical resources were found with name (%(name)s).")
@@ -249,8 +268,46 @@ class StackExists(HeatException):
     msg_fmt = _("The Stack (%(stack_name)s) already exists.")
 
 
-class StackValidationFailed(HeatException):
-    msg_fmt = _("%(message)s")
+class HeatExceptionWithPath(HeatException):
+    msg_fmt = _("%(error)s%(path)s%(message)s")
+
+    def __init__(self, error=None, path=None, message=None):
+        self.error = error or ''
+        self.path = []
+        if path is not None:
+            if isinstance(path, list):
+                self.path = path
+            elif isinstance(path, six.string_types):
+                self.path = [path]
+
+        result_path = ''
+        for path_item in self.path:
+            if isinstance(path_item, int) or path_item.isdigit():
+                result_path += '[%s]' % path_item
+            elif len(result_path) > 0:
+                result_path += '.%s' % path_item
+            else:
+                result_path = path_item
+
+        self.error_message = message or ''
+        super(HeatExceptionWithPath, self).__init__(
+            error=('%s: ' % self.error if self.error != '' else ''),
+            path=('%s: ' % result_path if len(result_path) > 0 else ''),
+            message=self.error_message
+        )
+
+    def error(self):
+        return self.error
+
+    def path(self):
+        return self.path
+
+    def error_message(self):
+        return self.error_message
+
+
+class StackValidationFailed(HeatExceptionWithPath):
+    pass
 
 
 class InvalidSchemaError(HeatException):
@@ -278,18 +335,63 @@ class WatchRuleNotFound(HeatException):
     msg_fmt = _("The Watch Rule (%(watch_name)s) could not be found.")
 
 
-class ResourceFailure(HeatException):
-    msg_fmt = _("%(exc_type)s: %(message)s")
-
-    def __init__(self, exception, resource, action=None):
-        if isinstance(exception, ResourceFailure):
-            exception = getattr(exception, 'exc', exception)
-        self.exc = exception
+class ResourceFailure(HeatExceptionWithPath):
+    def __init__(self, exception_or_error, resource, action=None):
         self.resource = resource
         self.action = action
-        exc_type = type(exception).__name__
-        super(ResourceFailure, self).__init__(exc_type=exc_type,
-                                              message=six.text_type(exception))
+        if action is None and resource is not None:
+            self.action = resource.action
+        path = []
+        res_path = []
+        if resource is not None:
+            res_path = [resource.stack.t.get_section_name('resources'),
+                        resource.name]
+
+        if isinstance(exception_or_error, Exception):
+            if isinstance(exception_or_error, ResourceFailure):
+                self.exc = exception_or_error.exc
+                error = exception_or_error.error
+                message = exception_or_error.error_message
+                path = exception_or_error.path
+            else:
+                self.exc = exception_or_error
+                error = six.text_type(type(self.exc).__name__)
+                message = six.text_type(self.exc)
+                path = res_path
+        else:
+            self.exc = None
+            res_failed = 'Resource %s failed: ' % action.upper()
+            if res_failed in exception_or_error:
+                (error, message, new_path) = self._from_status_reason(
+                    exception_or_error)
+                path = res_path + new_path
+            else:
+                path = res_path
+                error = None
+                message = exception_or_error
+
+        super(ResourceFailure, self).__init__(error=error, path=path,
+                                              message=message)
+
+    def _from_status_reason(self, status_reason):
+        """Split the status_reason up into parts.
+
+        Given the following status_reason:
+        "Resource DELETE failed: Exception : resources.AResource: foo"
+
+        we are going to return:
+        ("Exception", "resources.AResource", "foo")
+        """
+        parsed = [sp.strip() for sp in status_reason.split(':')]
+        if len(parsed) >= 4:
+            error = parsed[1]
+            message = ': '.join(parsed[3:])
+            path = parsed[2].split('.')
+        else:
+            error = ''
+            message = status_reason
+            path = []
+        return (error, message, path)
 
 
 class NotSupported(HeatException):
@@ -310,6 +412,16 @@ class ResourcePropertyConflict(HeatException):
         super(ResourcePropertyConflict, self).__init__(**kwargs)
 
 
+class PropertyUnspecifiedError(HeatException):
+    msg_fmt = _('At least one of the following properties '
+                'must be specified: %(props)s')
+
+    def __init__(self, *args, **kwargs):
+        if args:
+            kwargs.update({'props': ", ".join(args)})
+        super(PropertyUnspecifiedError, self).__init__(**kwargs)
+
+
 class HTTPExceptionDisguise(Exception):
     """Disguises HTTP exceptions so they can be handled by the webob fault
     application in the wsgi pipeline.
@@ -363,3 +475,31 @@ class StopActionFailed(HeatException):
 class EventSendFailed(HeatException):
     msg_fmt = _("Failed to send message to stack (%(stack_name)s) "
                 "on other engine (%(engine_id)s)")
+
+
+class ServiceNotFound(HeatException):
+    msg_fmt = _("Service %(service_id)s does not found")
+
+
+class UnsupportedObjectError(HeatException):
+    msg_fmt = _('Unsupported object type %(objtype)s')
+
+
+class OrphanedObjectError(HeatException):
+    msg_fmt = _('Cannot call %(method)s on orphaned %(objtype)s object')
+
+
+class IncompatibleObjectVersion(HeatException):
+    msg_fmt = _('Version %(objver)s of %(objname)s is not supported')
+
+
+class ObjectActionError(HeatException):
+    msg_fmt = _('Object action %(action)s failed because: %(reason)s')
+
+
+class ReadOnlyFieldError(HeatException):
+    msg_fmt = _('Cannot modify readonly field %(field)s')
+
+
+class ObjectFieldInvalid(HeatException):
+    msg_fmt = _('Field %(field)s of %(objname)s is not an instance of Field')
