@@ -12,16 +12,18 @@
 #    under the License.
 
 '''Implementation of SQLAlchemy backend.'''
-from datetime import datetime
-from datetime import timedelta
+import datetime
 import sys
 
-from oslo.config import cfg
-from oslo.db.sqlalchemy import session as db_session
-from oslo.db.sqlalchemy import utils
+from oslo_config import cfg
+from oslo_db.sqlalchemy import session as db_session
+from oslo_db.sqlalchemy import utils
+from oslo_utils import timeutils
+import osprofiler.sqlalchemy
+import six
 import sqlalchemy
 from sqlalchemy import orm
-from sqlalchemy.orm.session import Session
+from sqlalchemy.orm import session as orm_session
 
 from heat.common import crypt
 from heat.common import exception
@@ -33,6 +35,7 @@ from heat.rpc import api as rpc_api
 
 CONF = cfg.CONF
 CONF.import_opt('max_events_per_stack', 'heat.common.config')
+CONF.import_group('profiler', 'heat.common.config')
 
 _facade = None
 
@@ -42,6 +45,12 @@ def get_facade():
 
     if not _facade:
         _facade = db_session.EngineFacade.from_config(CONF)
+        if CONF.profiler.profiler_enabled:
+            if CONF.profiler.trace_sqlalchemy:
+                osprofiler.sqlalchemy.add_tracing(sqlalchemy,
+                                                  _facade.get_engine(),
+                                                  "db")
+
     return _facade
 
 get_engine = lambda: get_facade().get_engine()
@@ -115,10 +124,13 @@ def resource_get(context, resource_id):
 
 
 def resource_get_by_name_and_stack(context, resource_name, stack_id):
-    result = model_query(context, models.Resource).\
-        filter_by(name=resource_name).\
-        filter_by(stack_id=stack_id).\
-        options(orm.joinedload("data")).first()
+    result = model_query(
+        context, models.Resource
+    ).filter_by(
+        name=resource_name
+    ).filter_by(
+        stack_id=stack_id
+    ).options(orm.joinedload("data")).first()
     return result
 
 
@@ -142,6 +154,18 @@ def resource_get_all(context):
     return results
 
 
+def resource_update(context, resource_id, values, atomic_key,
+                    expected_engine_id=None):
+    session = _session(context)
+    with session.begin():
+        values['atomic_key'] = atomic_key + 1
+        rows_updated = session.query(models.Resource).filter_by(
+            id=resource_id, engine_id=expected_engine_id,
+            atomic_key=atomic_key).update(values)
+
+        return bool(rows_updated)
+
+
 def resource_data_get_all(resource, data=None):
     """
     Looks up resource_data by resource.id.  If data is encrypted,
@@ -250,9 +274,11 @@ def resource_create(context, values):
 
 
 def resource_get_all_by_stack(context, stack_id):
-    results = model_query(context, models.Resource).\
-        filter_by(stack_id=stack_id).\
-        options(orm.joinedload("data")).all()
+    results = model_query(
+        context, models.Resource
+    ).filter_by(
+        stack_id=stack_id
+    ).options(orm.joinedload("data")).all()
 
     if not results:
         raise exception.NotFound(_("no resources for stack_id %s were found")
@@ -261,23 +287,22 @@ def resource_get_all_by_stack(context, stack_id):
 
 
 def stack_get_by_name_and_owner_id(context, stack_name, owner_id):
-    query = soft_delete_aware_query(context, models.Stack).\
-        filter(sqlalchemy.or_(
-            models.Stack.tenant == context.tenant_id,
-            models.Stack.stack_user_project_id == context.tenant_id
-        )).\
-        filter_by(name=stack_name).\
-        filter_by(owner_id=owner_id)
+    query = soft_delete_aware_query(
+        context, models.Stack
+    ).filter(sqlalchemy.or_(
+             models.Stack.tenant == context.tenant_id,
+             models.Stack.stack_user_project_id == context.tenant_id)
+             ).filter_by(name=stack_name).filter_by(owner_id=owner_id)
     return query.first()
 
 
 def stack_get_by_name(context, stack_name):
-    query = soft_delete_aware_query(context, models.Stack).\
-        filter(sqlalchemy.or_(
-            models.Stack.tenant == context.tenant_id,
-            models.Stack.stack_user_project_id == context.tenant_id
-        )).\
-        filter_by(name=stack_name)
+    query = soft_delete_aware_query(
+        context, models.Stack
+    ).filter(sqlalchemy.or_(
+             models.Stack.tenant == context.tenant_id,
+             models.Stack.stack_user_project_id == context.tenant_id)
+             ).filter_by(name=stack_name)
     return query.first()
 
 
@@ -302,8 +327,8 @@ def stack_get(context, stack_id, show_deleted=False, tenant_safe=True,
 
 
 def stack_get_all_by_owner_id(context, owner_id):
-    results = soft_delete_aware_query(context, models.Stack).\
-        filter_by(owner_id=owner_id).all()
+    results = soft_delete_aware_query(
+        context, models.Stack).filter_by(owner_id=owner_id).all()
     return results
 
 
@@ -314,7 +339,7 @@ def _get_sort_keys(sort_keys, mapping):
     :param mapping: a mapping from keys to DB column names
     :returns: filtered list of sort keys
     '''
-    if isinstance(sort_keys, basestring):
+    if isinstance(sort_keys, six.string_types):
         sort_keys = [sort_keys]
     return [mapping[key] for key in sort_keys or [] if key in mapping]
 
@@ -345,13 +370,13 @@ def _paginate_query(context, query, model, limit=None, sort_keys=None,
 def _query_stack_get_all(context, tenant_safe=True, show_deleted=False,
                          show_nested=False):
     if show_nested:
-        query = soft_delete_aware_query(context, models.Stack,
-                                        show_deleted=show_deleted).\
-            filter_by(backup=False)
+        query = soft_delete_aware_query(
+            context, models.Stack, show_deleted=show_deleted
+        ).filter_by(backup=False)
     else:
-        query = soft_delete_aware_query(context, models.Stack,
-                                        show_deleted=show_deleted).\
-            filter_by(owner_id=None)
+        query = soft_delete_aware_query(
+            context, models.Stack, show_deleted=show_deleted
+        ).filter_by(owner_id=None)
 
     if tenant_safe:
         query = query.filter_by(tenant=context.tenant_id)
@@ -403,14 +428,21 @@ def stack_create(context, values):
 def stack_update(context, stack_id, values):
     stack = stack_get(context, stack_id)
 
-    if not stack:
+    if stack is None:
         raise exception.NotFound(_('Attempt to update a stack with id: '
                                  '%(id)s %(msg)s') % {
                                      'id': stack_id,
                                      'msg': 'that does not exist'})
 
-    stack.update(values)
-    stack.save(_session(context))
+    session = _session(context)
+    rows_updated = (session.query(models.Stack)
+                    .filter(models.Stack.id == stack.id)
+                    .filter(models.Stack.current_traversal
+                            == stack.current_traversal)
+                    .update(values, synchronize_session=False))
+    session.expire_all()
+
+    return (rows_updated is not None and rows_updated > 0)
 
 
 def stack_delete(context, stack_id):
@@ -420,7 +452,7 @@ def stack_delete(context, stack_id):
                                  '%(id)s %(msg)s') % {
                                      'id': stack_id,
                                      'msg': 'that does not exist'})
-    session = Session.object_session(s)
+    session = orm_session.Session.object_session(s)
 
     for r in s.resources:
         session.delete(r)
@@ -438,13 +470,22 @@ def stack_lock_create(stack_id, engine_id):
         session.add(models.StackLock(stack_id=stack_id, engine_id=engine_id))
 
 
+def stack_lock_get_engine_id(stack_id):
+    session = get_session()
+    with session.begin():
+        lock = session.query(models.StackLock).get(stack_id)
+        if lock is not None:
+            return lock.engine_id
+
+
 def stack_lock_steal(stack_id, old_engine_id, new_engine_id):
     session = get_session()
     with session.begin():
         lock = session.query(models.StackLock).get(stack_id)
-        rows_affected = session.query(models.StackLock).\
-            filter_by(stack_id=stack_id, engine_id=old_engine_id).\
-            update({"engine_id": new_engine_id})
+        rows_affected = session.query(
+            models.StackLock
+        ).filter_by(stack_id=stack_id, engine_id=old_engine_id
+                    ).update({"engine_id": new_engine_id})
     if not rows_affected:
         return lock.engine_id if lock is not None else True
 
@@ -452,13 +493,43 @@ def stack_lock_steal(stack_id, old_engine_id, new_engine_id):
 def stack_lock_release(stack_id, engine_id):
     session = get_session()
     with session.begin():
-        rows_affected = session.query(models.StackLock).\
-            filter_by(stack_id=stack_id, engine_id=engine_id).\
-            delete()
+        rows_affected = session.query(
+            models.StackLock
+        ).filter_by(stack_id=stack_id, engine_id=engine_id).delete()
     if not rows_affected:
         return True
 
 
+def stack_get_root_id(context, stack_id):
+    s = stack_get(context, stack_id)
+    while s.owner_id:
+        s = stack_get(context, s.owner_id)
+    return s.id
+
+
+def stack_count_total_resources(context, stack_id):
+
+    # start with a stack_get to confirm the context can access the stack
+    if stack_id is None or stack_get(context, stack_id) is None:
+        return 0
+
+    def nested_stack_ids(sid):
+        yield sid
+        for child in stack_get_all_by_owner_id(context, sid):
+            for stack in nested_stack_ids(child.id):
+                yield stack
+
+    stack_ids = list(nested_stack_ids(stack_id))
+
+    # count all resources which belong to the stacks
+    results = model_query(
+        context, models.Resource
+    ).filter(
+        models.Resource.stack_id.in_(stack_ids)
+    ).count()
+    return results
+
+
 def user_creds_create(context):
     values = context.to_dict()
     user_creds_ref = models.UserCreds()
@@ -471,9 +542,14 @@ def user_creds_create(context):
         user_creds_ref.password = None
         user_creds_ref.tenant = values.get('tenant')
         user_creds_ref.tenant_id = values.get('tenant_id')
+        user_creds_ref.auth_url = values.get('auth_url')
+        user_creds_ref.region_name = values.get('region_name')
     else:
         user_creds_ref.update(values)
         method, password = _encrypt(values['password'])
+        if len(six.text_type(password)) > 255:
+            raise exception.Error(_("Length of OS_PASSWORD after encryption"
+                                    " exceeds Heat limit (255 chars)"))
         user_creds_ref.password = password
         user_creds_ref.decrypt_method = method
     user_creds_ref.save(_session(context))
@@ -499,7 +575,7 @@ def user_creds_delete(context, user_creds_id):
         raise exception.NotFound(
             _('Attempt to delete user creds with id '
               '%(id)s that does not exist') % {'id': user_creds_id})
-    session = Session.object_session(creds)
+    session = orm_session.Session.object_session(creds)
     session.delete(creds)
     session.flush()
 
@@ -512,8 +588,9 @@ def event_get(context, event_id):
 def event_get_all(context):
     stacks = soft_delete_aware_query(context, models.Stack)
     stack_ids = [stack.id for stack in stacks]
-    results = model_query(context, models.Event).\
-        filter(models.Event.stack_id.in_(stack_ids)).all()
+    results = model_query(
+        context, models.Event
+    ).filter(models.Event.stack_id.in_(stack_ids)).all()
     return results
 
 
@@ -521,16 +598,16 @@ def event_get_all_by_tenant(context, limit=None, marker=None,
                             sort_keys=None, sort_dir=None, filters=None):
     query = model_query(context, models.Event)
     query = db_filters.exact_filter(query, models.Event, filters)
-    query = query.join(models.Event.stack).\
-        filter_by(tenant=context.tenant_id).filter_by(deleted_at=None)
+    query = query.join(
+        models.Event.stack
+    ).filter_by(tenant=context.tenant_id).filter_by(deleted_at=None)
     filters = None
     return _events_filter_and_page_query(context, query, limit, marker,
                                          sort_keys, sort_dir, filters).all()
 
 
 def _query_all_by_stack(context, stack_id):
-    query = model_query(context, models.Event).\
-        filter_by(stack_id=stack_id)
+    query = model_query(context, models.Event).filter_by(stack_id=stack_id)
     return query
 
 
@@ -557,8 +634,8 @@ def _events_paginate_query(context, query, model, limit=None, sort_keys=None,
     if marker:
         # not to use model_query(context, model).get(marker), because
         # user can only see the ID(column 'uuid') and the ID as the marker
-        model_marker = model_query(context, model).filter_by(uuid=marker).\
-            first()
+        model_marker = model_query(
+            context, model).filter_by(uuid=marker).first()
     try:
         query = utils.paginate_query(query, model, limit, sort_keys,
                                      model_marker, sort_dir)
@@ -623,8 +700,8 @@ def watch_rule_get(context, watch_rule_id):
 
 
 def watch_rule_get_by_name(context, watch_rule_name):
-    result = model_query(context, models.WatchRule).\
-        filter_by(name=watch_rule_name).first()
+    result = model_query(
+        context, models.WatchRule).filter_by(name=watch_rule_name).first()
     return result
 
 
@@ -634,8 +711,8 @@ def watch_rule_get_all(context):
 
 
 def watch_rule_get_all_by_stack(context, stack_id):
-    results = model_query(context, models.WatchRule).\
-        filter_by(stack_id=stack_id).all()
+    results = model_query(
+        context, models.WatchRule).filter_by(stack_id=stack_id).all()
     return results
 
 
@@ -665,7 +742,7 @@ def watch_rule_delete(context, watch_id):
                                  '%(id)s %(msg)s') % {
                                      'id': watch_id,
                                      'msg': 'that does not exist'})
-    session = Session.object_session(wr)
+    session = orm_session.Session.object_session(wr)
 
     for d in wr.watch_data:
         session.delete(d)
@@ -686,6 +763,12 @@ def watch_data_get_all(context):
     return results
 
 
+def watch_data_get_all_by_watch_rule_id(context, watch_rule_id):
+    results = model_query(context, models.WatchData).filter_by(
+        watch_rule_id=watch_rule_id).all()
+    return results
+
+
 def software_config_create(context, values):
     obj_ref = models.SoftwareConfig()
     obj_ref.update(values)
@@ -707,7 +790,7 @@ def software_config_get(context, config_id):
 
 def software_config_delete(context, config_id):
     config = software_config_get(context, config_id)
-    session = Session.object_session(config)
+    session = orm_session.Session.object_session(config)
     session.delete(config)
     session.flush()
 
@@ -734,12 +817,12 @@ def software_deployment_get(context, deployment_id):
 
 def software_deployment_get_all(context, server_id=None):
     sd = models.SoftwareDeployment
-    query = model_query(context, sd).\
-        filter(sqlalchemy.or_(
-            sd.tenant == context.tenant_id,
-            sd.stack_user_project_id == context.tenant_id
-        )).\
-        order_by(sd.created_at)
+    query = model_query(
+        context, sd
+    ).filter(sqlalchemy.or_(
+             sd.tenant == context.tenant_id,
+             sd.stack_user_project_id == context.tenant_id)
+             ).order_by(sd.created_at)
     if server_id:
         query = query.filter_by(server_id=server_id)
     return query.all()
@@ -754,7 +837,7 @@ def software_deployment_update(context, deployment_id, values):
 
 def software_deployment_delete(context, deployment_id):
     deployment = software_deployment_get(context, deployment_id)
-    session = Session.object_session(deployment)
+    session = orm_session.Session.object_session(deployment)
     session.delete(deployment)
     session.flush()
 
@@ -787,7 +870,7 @@ def snapshot_update(context, snapshot_id, values):
 
 def snapshot_delete(context, snapshot_id):
     snapshot = snapshot_get(context, snapshot_id)
-    session = Session.object_session(snapshot)
+    session = orm_session.Session.object_session(snapshot)
     session.delete(snapshot)
     session.flush()
 
@@ -797,6 +880,50 @@ def snapshot_get_all(context, stack_id):
         stack_id=stack_id, tenant=context.tenant_id)
 
 
+def service_create(context, values):
+    service = models.Service()
+    service.update(values)
+    service.save(_session(context))
+    return service
+
+
+def service_update(context, service_id, values):
+    service = service_get(context, service_id)
+    values.update({'updated_at': timeutils.utcnow()})
+    service.update(values)
+    service.save(_session(context))
+    return service
+
+
+def service_delete(context, service_id, soft_delete=True):
+    service = service_get(context, service_id)
+    session = orm_session.Session.object_session(service)
+    if soft_delete:
+        service.soft_delete(session=session)
+    else:
+        session.delete(service)
+    session.flush()
+
+
+def service_get(context, service_id):
+    result = model_query(context, models.Service).get(service_id)
+    if result is None:
+        raise exception.ServiceNotFound(service_id=service_id)
+    return result
+
+
+def service_get_all(context):
+    return (model_query(context, models.Service).
+            filter_by(deleted_at=None).all())
+
+
+def service_get_all_by_args(context, host, binary, hostname):
+    return (model_query(context, models.Service).
+            filter_by(host=host).
+            filter_by(binary=binary).
+            filter_by(hostname=hostname).all())
+
+
 def purge_deleted(age, granularity='days'):
     try:
         age = int(age)
@@ -816,20 +943,22 @@ def purge_deleted(age, granularity='days'):
     elif granularity == 'minutes':
         age = age * 60
 
-    time_line = datetime.now() - timedelta(seconds=age)
+    time_line = datetime.datetime.now() - datetime.timedelta(seconds=age)
     engine = get_engine()
     meta = sqlalchemy.MetaData()
     meta.bind = engine
 
+    # Purge deleted stacks
     stack = sqlalchemy.Table('stack', meta, autoload=True)
     event = sqlalchemy.Table('event', meta, autoload=True)
     raw_template = sqlalchemy.Table('raw_template', meta, autoload=True)
     user_creds = sqlalchemy.Table('user_creds', meta, autoload=True)
 
-    stmt = sqlalchemy.select([stack.c.id,
-                              stack.c.raw_template_id,
-                              stack.c.user_creds_id]).\
-        where(stack.c.deleted_at < time_line)
+    stmt = sqlalchemy.select(
+        [stack.c.id,
+         stack.c.raw_template_id,
+         stack.c.user_creds_id]
+    ).where(stack.c.deleted_at < time_line)
     deleted_stacks = engine.execute(stmt)
 
     for s in deleted_stacks:
@@ -837,12 +966,54 @@ def purge_deleted(age, granularity='days'):
         engine.execute(event_del)
         stack_del = stack.delete().where(stack.c.id == s[0])
         engine.execute(stack_del)
-        raw_template_del = raw_template.delete().\
-            where(raw_template.c.id == s[1])
+        raw_template_del = raw_template.delete().where(
+            raw_template.c.id == s[1])
         engine.execute(raw_template_del)
         user_creds_del = user_creds.delete().where(user_creds.c.id == s[2])
         engine.execute(user_creds_del)
 
+    # Purge deleted services
+    service = sqlalchemy.Table('service', meta, autoload=True)
+    stmt = (sqlalchemy.select([service.c.id]).
+            where(service.c.deleted_at < time_line))
+    deleted_services = engine.execute(stmt)
+
+    for s in deleted_services:
+        stmt = service.delete().where(service.c.id == s[0])
+        engine.execute(stmt)
+
+
+def sync_point_delete_all_by_stack_and_traversal(context, stack_id,
+                                                 traversal_id):
+    rows_deleted = model_query(context, models.SyncPoint).filter_by(
+        stack_id=stack_id, traversal_id=traversal_id).delete()
+    return rows_deleted
+
+
+def sync_point_create(context, values):
+    sync_point_ref = models.SyncPoint()
+    sync_point_ref.update(values)
+    sync_point_ref.save(_session(context))
+    return sync_point_ref
+
+
+def sync_point_get(context, entity_id, traversal_id, is_update):
+    return model_query(context, models.SyncPoint).get(
+        (entity_id, traversal_id, is_update)
+    )
+
+
+def sync_point_update_input_data(context, entity_id,
+                                 traversal_id, is_update, atomic_key,
+                                 input_data):
+    rows_updated = model_query(context, models.SyncPoint).filter_by(
+        entity_id=entity_id,
+        traversal_id=traversal_id,
+        is_update=is_update,
+        atomic_key=atomic_key
+    ).update({"input_data": input_data, "atomic_key": atomic_key + 1})
+    return rows_updated
+
 
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
