@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,11 +14,11 @@
 import collections
 import itertools
 
-from six.moves import xrange
+from oslo_utils import encodeutils
+import six
 
 from heat.common import exception
-
-from heat.openstack.common.gettextutils import _
+from heat.common.i18n import _
 
 
 class CircularDependencyException(exception.HeatException):
@@ -85,7 +84,13 @@ class Node(object):
 
     def __str__(self):
         '''Return a human-readable string representation of the node.'''
-        return '{%s}' % ', '.join(str(n) for n in self)
+        text = '{%s}' % ', '.join(str(n) for n in self)
+        return encodeutils.safe_encode(text)
+
+    def __unicode__(self):
+        '''Return a human-readable string representation of the node.'''
+        text = '{%s}' % ', '.join(six.text_type(n) for n in self)
+        return encodeutils.safe_decode(text)
 
     def __repr__(self):
         '''Return a string representation of the node.'''
@@ -122,7 +127,7 @@ class Graph(collections.defaultdict):
                 for rqd in node:
                     yield (rqr, rqd)
         return itertools.chain.from_iterable(outgoing_edges(*i)
-                                             for i in self.iteritems())
+                                             for i in six.iteritems(self))
 
     def __delitem__(self, key):
         '''Delete the node given by the specified key from the graph.'''
@@ -137,8 +142,16 @@ class Graph(collections.defaultdict):
 
     def __str__(self):
         '''Convert the graph to a human-readable string.'''
-        pairs = ('%s: %s' % (str(k), str(v)) for k, v in self.iteritems())
-        return '{%s}' % ', '.join(pairs)
+        pairs = ('%s: %s' % (str(k), str(v)) for k, v in six.iteritems(self))
+        text = '{%s}' % ', '.join(pairs)
+        return encodeutils.safe_encode(text)
+
+    def __unicode__(self):
+        '''Convert the graph to a human-readable string.'''
+        pairs = ('%s: %s' % (six.text_type(k), six.text_type(v))
+                 for k, v in six.iteritems(self))
+        text = '{%s}' % ', '.join(pairs)
+        return encodeutils.safe_decode(text)
 
     @staticmethod
     def toposort(graph):
@@ -147,8 +160,8 @@ class Graph(collections.defaultdict):
 
         This is a destructive operation for the graph.
         '''
-        for iteration in xrange(len(graph)):
-            for key, node in graph.iteritems():
+        for iteration in six.moves.xrange(len(graph)):
+            for key, node in six.iteritems(graph):
                 if not node:
                     yield key
                     del graph[key]
@@ -156,17 +169,18 @@ class Graph(collections.defaultdict):
             else:
                 # There are nodes remaining, but none without
                 # dependencies: a cycle
-                raise CircularDependencyException(cycle=str(graph))
+                raise CircularDependencyException(cycle=six.text_type(graph))
 
 
 class Dependencies(object):
     '''Helper class for calculating a dependency graph.'''
 
-    def __init__(self, edges=[]):
+    def __init__(self, edges=None):
         '''
         Initialise, optionally with a list of edges, in the form of
         (requirer, required) tuples.
         '''
+        edges = edges or []
         self._graph = Graph()
         for e in edges:
             self += e
@@ -227,10 +241,17 @@ class Dependencies(object):
         '''
         return str(self._graph)
 
+    def __unicode__(self):
+        '''
+        Return a human-readable string representation of the dependency graph
+        '''
+        return six.text_type(self._graph)
+
     def __repr__(self):
         '''Return a string representation of the object.'''
         edge_reprs = (repr(e) for e in self._graph.edges())
-        return 'Dependencies([%s])' % ', '.join(edge_reprs)
+        text = 'Dependencies([%s])' % ', '.join(edge_reprs)
+        return encodeutils.safe_encode(text)
 
     def graph(self, reverse=False):
         '''Return a copy of the underlying dependency graph.'''
@@ -240,9 +261,9 @@ class Dependencies(object):
             return self._graph.copy()
 
     def __iter__(self):
-        '''Return a topologically sorted iterator'''
+        '''Return a topologically sorted iterator.'''
         return Graph.toposort(self.graph())
 
     def __reversed__(self):
-        '''Return a reverse topologically sorted iterator'''
+        '''Return a reverse topologically sorted iterator.'''
         return Graph.toposort(self.graph(reverse=True))
