@@ -11,8 +11,10 @@
 #    under the License.
 
 import collections
+
 import six
 
+from heat.common import exception
 from heat.common.i18n import _
 from heat.engine.cfn import functions as cfn_funcs
 from heat.engine.cfn import template as cfn_template
@@ -37,10 +39,13 @@ class HOTemplate20130523(template.Template):
     A Heat Orchestration Template format stack template.
     """
 
-    SECTIONS = (VERSION, DESCRIPTION, PARAMETER_GROUPS, PARAMETERS,
-                RESOURCES, OUTPUTS, MAPPINGS) = \
-               ('heat_template_version', 'description', 'parameter_groups',
-                'parameters', 'resources', 'outputs', '__undefined__')
+    SECTIONS = (
+        VERSION, DESCRIPTION, PARAMETER_GROUPS,
+        PARAMETERS, RESOURCES, OUTPUTS, MAPPINGS
+    ) = (
+        'heat_template_version', 'description', 'parameter_groups',
+        'parameters', 'resources', 'outputs', '__undefined__'
+    )
 
     SECTIONS_NO_DIRECT_ACCESS = set([PARAMETERS, VERSION])
 
@@ -51,6 +56,12 @@ class HOTemplate20130523(template.Template):
                             cfn_template.CfnTemplate.RESOURCES: RESOURCES,
                             cfn_template.CfnTemplate.OUTPUTS: OUTPUTS}
 
+    _RESOURCE_HOT_TO_CFN_ATTRS = {'type': 'Type',
+                                  'properties': 'Properties',
+                                  'metadata': 'Metadata',
+                                  'depends_on': 'DependsOn',
+                                  'deletion_policy': 'DeletionPolicy',
+                                  'update_policy': 'UpdatePolicy'}
     functions = {
         'Fn::GetAZs': cfn_funcs.GetAZs,
         'get_param': hot_funcs.GetParam,
@@ -72,7 +83,7 @@ class HOTemplate20130523(template.Template):
 
     def __getitem__(self, section):
         """"Get the relevant section in the template."""
-        #first translate from CFN into HOT terminology if necessary
+        # first translate from CFN into HOT terminology if necessary
         if section not in self.SECTIONS:
             section = HOTemplate20130523._translate(
                 section, self._CFN_TO_HOT_SECTIONS,
@@ -118,125 +129,139 @@ class HOTemplate20130523(template.Template):
             else:
                 raise ke
 
+    def _translate_section(self, section, sub_section, data, mapping):
+        cfn_objects = {}
+        obj_name = section[:-1]
+        err_msg = _('"%%s" is not a valid keyword inside a %s '
+                    'definition') % obj_name
+        for name, attrs in six.iteritems(data):
+            cfn_object = {}
+
+            if not attrs:
+                args = {'object_name': obj_name, 'sub_section': sub_section}
+                message = _('Each %(object_name)s must contain a '
+                            '%(sub_section)s key.') % args
+                raise exception.StackValidationFailed(message=message)
+            try:
+                for attr, attr_value in six.iteritems(attrs):
+                    cfn_attr = self._translate(attr, mapping, err_msg)
+                    cfn_object[cfn_attr] = attr_value
+
+                cfn_objects[name] = cfn_object
+            except AttributeError:
+                message = _('"%(section)s" must contain a map of '
+                            '%(obj_name)s maps. Found a [%(_type)s] '
+                            'instead') % {'section': section,
+                                          '_type': type(attrs),
+                                          'obj_name': obj_name}
+                raise exception.StackValidationFailed(message=message)
+            except KeyError as e:
+                # an invalid keyword was found
+                raise exception.StackValidationFailed(message=six.text_type(e))
+
+        return cfn_objects
+
     def _translate_resources(self, resources):
         """Get the resources of the template translated into CFN format."""
-        HOT_TO_CFN_ATTRS = {'type': 'Type',
-                            'properties': 'Properties',
-                            'metadata': 'Metadata',
-                            'depends_on': 'DependsOn',
-                            'deletion_policy': 'DeletionPolicy',
-                            'update_policy': 'UpdatePolicy'}
 
-        cfn_resources = {}
+        return self._translate_section('resources', 'type', resources,
+                                       self._RESOURCE_HOT_TO_CFN_ATTRS)
 
-        for resource_name, attrs in six.iteritems(resources):
-            cfn_resource = {}
-
-            for attr, attr_value in six.iteritems(attrs):
-                cfn_attr = self._translate(attr, HOT_TO_CFN_ATTRS,
-                                           _('"%s" is not a valid keyword '
-                                             'inside a resource definition'))
-                cfn_resource[cfn_attr] = attr_value
-
-            cfn_resources[resource_name] = cfn_resource
-
-        return cfn_resources
+    def get_section_name(self, section):
+        cfn_to_hot_attrs = dict(zip(self._RESOURCE_HOT_TO_CFN_ATTRS.values(),
+                                    self._RESOURCE_HOT_TO_CFN_ATTRS.keys()))
+        return cfn_to_hot_attrs.get(section, section)
 
     def _translate_outputs(self, outputs):
         """Get the outputs of the template translated into CFN format."""
         HOT_TO_CFN_ATTRS = {'description': 'Description',
                             'value': 'Value'}
 
-        cfn_outputs = {}
+        return self._translate_section('outputs', 'value', outputs,
+                                       HOT_TO_CFN_ATTRS)
 
-        for output_name, attrs in six.iteritems(outputs):
-            cfn_output = {}
+    def param_schemata(self, param_defaults=None):
+        parameter_section = self.t.get(self.PARAMETERS) or {}
+        pdefaults = param_defaults or {}
+        for name, schema in six.iteritems(parameter_section):
+            if name in pdefaults:
+                parameter_section[name]['default'] = pdefaults[name]
 
-            for attr, attr_value in six.iteritems(attrs):
-                cfn_attr = self._translate(attr, HOT_TO_CFN_ATTRS,
-                                           _('"%s" is not a valid keyword '
-                                             'inside an output definition'))
-                cfn_output[cfn_attr] = attr_value
-
-            cfn_outputs[output_name] = cfn_output
-
-        return cfn_outputs
-
-    def param_schemata(self):
-        parameter_section = self.t.get(self.PARAMETERS)
-        if parameter_section is None:
-            parameter_section = {}
         params = six.iteritems(parameter_section)
         return dict((name, parameters.HOTParamSchema.from_dict(name, schema))
                     for name, schema in params)
 
-    def parameters(self, stack_identifier, user_params):
+    def parameters(self, stack_identifier, user_params, param_defaults=None):
         return parameters.HOTParameters(stack_identifier, self,
-                                        user_params=user_params)
+                                        user_params=user_params,
+                                        param_defaults=param_defaults)
+
+    def validate_resource_definitions(self, stack):
+        resources = self.t.get(self.RESOURCES) or {}
+        allowed_keys = set(_RESOURCE_KEYS)
+
+        try:
+            for name, snippet in resources.items():
+                data = self.parse(stack, snippet)
+
+                if not self.validate_resource_key_type(RES_TYPE,
+                                                       six.string_types,
+                                                       'string',
+                                                       allowed_keys,
+                                                       name, data):
+                    args = {'name': name, 'type_key': RES_TYPE}
+                    msg = _('Resource %(name)s is missing '
+                            '"%(type_key)s"') % args
+                    raise KeyError(msg)
+
+                self.validate_resource_key_type(
+                    RES_PROPERTIES,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_METADATA,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_DEPENDS_ON,
+                    collections.Sequence,
+                    'list or string', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_DELETION_POLICY,
+                    six.string_types,
+                    'string', allowed_keys, name, data)
+                self.validate_resource_key_type(
+                    RES_UPDATE_POLICY,
+                    (collections.Mapping, function.Function),
+                    'object', allowed_keys, name, data)
+        except (TypeError, ValueError) as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
 
     def resource_definitions(self, stack):
-        allowed_keys = set(_RESOURCE_KEYS)
+        resources = self.t.get(self.RESOURCES) or {}
 
         def rsrc_defn_item(name, snippet):
             data = self.parse(stack, snippet)
 
-            def get_check_type(key, valid_types, typename, default=None):
-                if key in data:
-                    field = data[key]
-                    if not isinstance(field, valid_types):
-                        args = {'name': name, 'key': key, 'typename': typename}
-                        msg = _('Resource %(name)s %(key)s type'
-                                'must be %(typename)s') % args
-                        raise TypeError(msg)
-                    return field
-                else:
-                    return default
-
-            resource_type = get_check_type(RES_TYPE, basestring, 'string')
-            if resource_type is None:
-                args = {'name': name, 'type_key': RES_TYPE}
-                msg = _('Resource %(name)s is missing "%(type_key)s"') % args
-                raise KeyError(msg)
-
-            properties = get_check_type(RES_PROPERTIES,
-                                        (collections.Mapping,
-                                         function.Function),
-                                        'object')
-
-            metadata = get_check_type(RES_METADATA,
-                                      (collections.Mapping,
-                                       function.Function),
-                                      'object')
-
-            depends = get_check_type(RES_DEPENDS_ON,
-                                     collections.Sequence,
-                                     'list or string',
-                                     default=None)
-            if isinstance(depends, basestring):
+            depends = data.get(RES_DEPENDS_ON)
+            if not depends:
+                depends = []
+            elif isinstance(depends, six.string_types):
                 depends = [depends]
 
-            deletion_policy = get_check_type(RES_DELETION_POLICY,
-                                             basestring,
-                                             'string')
+            kwargs = {
+                'resource_type': data.get(RES_TYPE),
+                'properties': data.get(RES_PROPERTIES),
+                'metadata': data.get(RES_METADATA),
+                'depends': depends,
+                'deletion_policy': data.get(RES_DELETION_POLICY),
+                'update_policy': data.get(RES_UPDATE_POLICY),
+                'description': None
+            }
 
-            update_policy = get_check_type(RES_UPDATE_POLICY,
-                                           (collections.Mapping,
-                                            function.Function),
-                                           'object')
-
-            for key in data:
-                if key not in allowed_keys:
-                    raise ValueError(_('"%s" is not a valid keyword '
-                                       'inside a resource definition') % key)
-
-            defn = rsrc_defn.ResourceDefinition(name, resource_type,
-                                                properties, metadata,
-                                                depends,
-                                                deletion_policy,
-                                                update_policy)
+            defn = rsrc_defn.ResourceDefinition(name, **kwargs)
             return name, defn
 
-        resources = self.t.get(self.RESOURCES) or {}
         return dict(rsrc_defn_item(name, data)
                     for name, data in resources.items())
 
@@ -271,3 +296,29 @@ class HOTemplate20141016(HOTemplate20130523):
         'Fn::ResourceFacade': hot_funcs.Removed,
         'Ref': hot_funcs.Removed,
     }
+
+
+class HOTemplate20150430(HOTemplate20141016):
+    functions = {
+        'digest': hot_funcs.Digest,
+        'get_attr': hot_funcs.GetAtt,
+        'get_file': hot_funcs.GetFile,
+        'get_param': hot_funcs.GetParam,
+        'get_resource': cfn_funcs.ResourceRef,
+        'list_join': hot_funcs.Join,
+        'repeat': hot_funcs.Repeat,
+        'resource_facade': hot_funcs.ResourceFacade,
+        'str_replace': hot_funcs.Replace,
+
+        'Fn::Select': cfn_funcs.Select,
+
+        # functions removed from 20130523
+        'Fn::GetAZs': hot_funcs.Removed,
+        'Fn::Join': hot_funcs.Removed,
+        'Fn::Split': hot_funcs.Removed,
+        'Fn::Replace': hot_funcs.Removed,
+        'Fn::Base64': hot_funcs.Removed,
+        'Fn::MemberListToMap': hot_funcs.Removed,
+        'Fn::ResourceFacade': hot_funcs.Removed,
+        'Ref': hot_funcs.Removed,
+    }
