@@ -13,18 +13,24 @@
 
 import base64
 import contextlib
-from datetime import datetime
-from oslo.config import cfg
-from oslo.utils import excutils
-import six
+import datetime as dt
 import warnings
+import weakref
+
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import encodeutils
+from oslo_utils import excutils
+import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LE
+from heat.common.i18n import _LI
+from heat.common.i18n import _LW
 from heat.common import identifier
 from heat.common import short_id
 from heat.common import timeutils
-from heat.db import api as db_api
 from heat.engine import attributes
 from heat.engine import environment
 from heat.engine import event
@@ -34,21 +40,15 @@ from heat.engine import resources
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.engine import support
-from heat.openstack.common import log as logging
+from heat.objects import resource as resource_objects
+from heat.objects import resource_data as resource_data_objects
+from heat.rpc import client as rpc_client
 
 cfg.CONF.import_opt('action_retry_limit', 'heat.common.config')
 
 LOG = logging.getLogger(__name__)
 
-
-def get_types(support_status):
-    '''Return a list of valid resource types.'''
-    return resources.global_env().get_types(support_status)
-
-
-def get_class(resource_type, resource_name=None):
-    '''Return the Resource class for a given resource type.'''
-    return resources.global_env().get_class(resource_type, resource_name)
+datetime = dt.datetime
 
 
 def _register_class(resource_type, resource_class):
@@ -72,10 +72,13 @@ class ResourceInError(exception.HeatException):
 
 
 class ResourceUnknownStatus(exception.HeatException):
-    msg_fmt = _('%(result)s - Unknown status %(resource_status)s')
+    msg_fmt = _('%(result)s - Unknown status %(resource_status)s due to '
+                '"%(status_reason)s"')
 
-    def __init__(self, result=_('Resource failed'), **kwargs):
-        super(ResourceUnknownStatus, self).__init__(result=result, **kwargs)
+    def __init__(self, result=_('Resource failed'),
+                 status_reason=_('Unknown'), **kwargs):
+        super(ResourceUnknownStatus, self).__init__(
+            result=result, status_reason=status_reason, **kwargs)
 
 
 class Resource(object):
@@ -124,32 +127,13 @@ class Resource(object):
             ResourceClass = cls
         else:
             from heat.engine.resources import template_resource
-            # Select the correct subclass to instantiate.
-
-            # Note: If the current stack is an implementation of
-            # a resource type (a TemplateResource mapped in the environment)
-            # then don't infinitely recurse by creating a child stack
-            # of the same type. Instead get the next match which will get
-            # us closer to a concrete class.
-            def get_ancestor_template_resources():
-                """Return an ancestry list (TemplateResources only)."""
-                parent = stack.parent_resource
-                while parent is not None:
-                    if isinstance(parent, template_resource.TemplateResource):
-                        yield parent.template_name
-                    parent = parent.stack.parent_resource
-
-            ancestor_list = set(get_ancestor_template_resources())
-
-            def accept_class(res_info):
-                if not isinstance(res_info, environment.TemplateResourceInfo):
-                    return True
-                return res_info.template_name not in ancestor_list
 
             registry = stack.env.registry
-            ResourceClass = registry.get_class(definition.resource_type,
-                                               resource_name=name,
-                                               accept_fn=accept_class)
+            try:
+                ResourceClass = registry.get_class(definition.resource_type,
+                                                   resource_name=name)
+            except exception.NotFound:
+                ResourceClass = template_resource.TemplateResource
             assert issubclass(ResourceClass, Resource)
 
         return super(Resource, cls).__new__(ResourceClass)
@@ -182,16 +166,29 @@ class Resource(object):
         self.status = self.COMPLETE
         self.status_reason = ''
         self.id = None
+        self.uuid = None
         self._data = {}
         self._rsrc_metadata = None
         self._stored_properties_data = None
         self.created_time = None
         self.updated_time = None
+        self._rpc_client = None
+        self.needed_by = None
+        self.requires = None
+        self.replaces = None
+        self.replaced_by = None
+        self.current_template_id = stack.t.id
 
         resource = stack.db_resource_get(name)
         if resource:
             self._load_data(resource)
 
+    def rpc_client(self):
+        '''Return a client for making engine RPC calls.'''
+        if not self._rpc_client:
+            self._rpc_client = rpc_client.EngineClient()
+        return self._rpc_client
+
     def _load_data(self, resource):
         '''Load the resource state from its DB representation.'''
         self.resource_id = resource.nova_instance
@@ -199,14 +196,31 @@ class Resource(object):
         self.status = resource.status
         self.status_reason = resource.status_reason
         self.id = resource.id
+        self.uuid = resource.uuid
         try:
-            self._data = db_api.resource_data_get_all(self, resource.data)
+            self._data = resource_data_objects.ResourceData.get_all(
+                self, resource.data)
         except exception.NotFound:
             self._data = {}
         self._rsrc_metadata = resource.rsrc_metadata
         self._stored_properties_data = resource.properties_data
         self.created_time = resource.created_at
         self.updated_time = resource.updated_at
+        self.needed_by = resource.needed_by
+        self.requires = resource.requires
+        self.replaces = resource.replaces
+        self.replaced_by = resource.replaced_by
+        self.current_template_id = resource.current_template_id
+
+    @property
+    def stack(self):
+        stack = self._stackref()
+        assert stack is not None, "Need a reference to the Stack object"
+        return stack
+
+    @stack.setter
+    def stack(self, stack):
+        self._stackref = weakref.ref(stack)
 
     def reparse(self):
         self.properties = self.t.properties(self.properties_schema,
@@ -247,22 +261,50 @@ class Resource(object):
     def metadata_get(self, refresh=False):
         if refresh:
             self._rsrc_metadata = None
-        if self.id is None:
+        if self.id is None or self.action == self.INIT:
             return self.t.metadata()
         if self._rsrc_metadata is not None:
             return self._rsrc_metadata
-        rs = db_api.resource_get(self.stack.context, self.id)
+        rs = resource_objects.Resource.get_obj(self.stack.context, self.id)
         rs.refresh(attrs=['rsrc_metadata'])
         self._rsrc_metadata = rs.rsrc_metadata
         return rs.rsrc_metadata
 
     def metadata_set(self, metadata):
-        if self.id is None:
+        if self.id is None or self.action == self.INIT:
             raise exception.ResourceNotAvailable(resource_name=self.name)
-        rs = db_api.resource_get(self.stack.context, self.id)
+        rs = resource_objects.Resource.get_obj(self.stack.context, self.id)
         rs.update_and_save({'rsrc_metadata': metadata})
         self._rsrc_metadata = metadata
 
+    def _break_if_required(self, action, hook):
+        '''Block the resource until the hook is cleared if there is one.'''
+        if self.stack.env.registry.matches_hook(self.name, hook):
+            self._add_event(self.action, self.status,
+                            _("%(a)s paused until Hook %(h)s is cleared")
+                            % {'a': action, 'h': hook})
+            self.trigger_hook(hook)
+            LOG.info(_LI('Reached hook on %s'), six.text_type(self))
+        while self.has_hook(hook) and self.status != self.FAILED:
+            try:
+                yield
+            except Exception:
+                self.clear_hook(hook)
+                self._add_event(
+                    self.action, self.status,
+                    "Failure occured while waiting.")
+
+    def has_hook(self, hook):
+        # Clear the cache to make sure the data is up to date:
+        self._data = None
+        return self.data().get(hook) == "True"
+
+    def trigger_hook(self, hook):
+        self.data_set(hook, "True")
+
+    def clear_hook(self, hook):
+        self.data_delete(hook)
+
     def type(self):
         return self.t.resource_type
 
@@ -366,11 +408,30 @@ class Resource(object):
     def __str__(self):
         if self.stack.id:
             if self.resource_id:
-                return '%s "%s" [%s] %s' % (self.__class__.__name__, self.name,
+                text = '%s "%s" [%s] %s' % (self.__class__.__name__, self.name,
                                             self.resource_id, str(self.stack))
-            return '%s "%s" %s' % (self.__class__.__name__, self.name,
-                                   str(self.stack))
-        return '%s "%s"' % (self.__class__.__name__, self.name)
+            else:
+                text = '%s "%s" %s' % (self.__class__.__name__, self.name,
+                                       str(self.stack))
+        else:
+            text = '%s "%s"' % (self.__class__.__name__, self.name)
+        return encodeutils.safe_encode(text)
+
+    def __unicode__(self):
+        if self.stack.id:
+            if self.resource_id:
+                text = '%s "%s" [%s] %s' % (self.__class__.__name__, self.name,
+                                            self.resource_id,
+                                            six.text_type(self.stack))
+            else:
+                text = '%s "%s" %s' % (self.__class__.__name__, self.name,
+                                       six.text_type(self.stack))
+        else:
+            text = '%s "%s"' % (self.__class__.__name__, self.name)
+        return encodeutils.safe_decode(text)
+
+    def dep_attrs(self, resource_name):
+        return self.t.dep_attrs(resource_name)
 
     def add_dependencies(self, deps):
         for dep in self.t.dependencies(self.stack):
@@ -442,7 +503,7 @@ class Resource(object):
                 LOG.debug('%s', six.text_type(ex))
         except Exception as ex:
             LOG.info('%(action)s: %(info)s', {"action": action,
-                                              "info": str(self)},
+                                              "info": six.text_type(self)},
                      exc_info=True)
             failure = exception.ResourceFailure(ex, self, action)
             self.state_set(action, self.FAILED, six.text_type(failure))
@@ -452,7 +513,7 @@ class Resource(object):
                 try:
                     self.state_set(action, self.FAILED, '%s aborted' % action)
                 except Exception:
-                    LOG.exception(_('Error marking resource as failed'))
+                    LOG.exception(_LE('Error marking resource as failed'))
         else:
             self.state_set(action, self.COMPLETE)
 
@@ -529,10 +590,17 @@ class Resource(object):
         action = self.CREATE
         if (self.action, self.status) != (self.INIT, self.COMPLETE):
             exc = exception.Error(_('State %s invalid for create')
-                                  % str(self.state))
+                                  % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
 
-        LOG.info(_('creating %s') % str(self))
+        # This method can be called when we replace a resource, too. In that
+        # case, a hook has already been dealt with in `Resource.update` so we
+        # shouldn't do it here again:
+        if self.stack.action == self.stack.CREATE:
+            yield self._break_if_required(
+                self.CREATE, environment.HOOK_PRE_CREATE)
+
+        LOG.info(_LI('creating %s'), six.text_type(self))
 
         # Re-resolve the template, since if the resource Ref's
         # the StackId pseudo parameter, it will change after
@@ -668,9 +736,14 @@ class Resource(object):
 
         before_props = before.properties(self.properties_schema,
                                          self.context)
+        # Regenerate the schema, else validation would fail
+        self.regenerate_info_schema(after)
         after_props = after.properties(self.properties_schema,
                                        self.context)
 
+        yield self._break_if_required(
+            self.UPDATE, environment.HOOK_PRE_UPDATE)
+
         if not self._needs_update(after, before, after_props, before_props,
                                   prev_resource):
             return
@@ -681,7 +754,7 @@ class Resource(object):
             exc = Exception(_('Resource update already requested'))
             raise exception.ResourceFailure(exc, self, action)
 
-        LOG.info(_('updating %s') % str(self))
+        LOG.info(_LI('updating %s'), six.text_type(self))
 
         self.updated_time = datetime.utcnow()
         with self._action_recorder(action, UpdateReplace):
@@ -706,7 +779,7 @@ class Resource(object):
         original state with the added message that check was not performed.
         """
         action = self.CHECK
-        LOG.info(_('Checking %s') % six.text_type(self))
+        LOG.info(_LI('Checking %s'), six.text_type(self))
 
         if hasattr(self, 'handle_%s' % action.lower()):
             return self._do_action(action)
@@ -714,6 +787,22 @@ class Resource(object):
             reason = '%s not supported for %s' % (action, self.type())
             self.state_set(action, self.COMPLETE, reason)
 
+    def _verify_check_conditions(self, checks):
+        def valid(check):
+            if isinstance(check['expected'], list):
+                return check['current'] in check['expected']
+            else:
+                return check['current'] == check['expected']
+
+        msg = _("'%(attr)s': expected '%(expected)s', got '%(current)s'")
+        invalid_checks = [
+            msg % check
+            for check in checks
+            if not valid(check)
+        ]
+        if invalid_checks:
+            raise exception.Error('; '.join(invalid_checks))
+
     def suspend(self):
         '''
         Suspend the resource.  Subclasses should provide a handle_suspend()
@@ -724,10 +813,10 @@ class Resource(object):
         # Don't try to suspend the resource unless it's in a stable state
         if (self.action == self.DELETE or self.status != self.COMPLETE):
             exc = exception.Error(_('State %s invalid for suspend')
-                                  % str(self.state))
+                                  % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
 
-        LOG.info(_('suspending %s') % str(self))
+        LOG.info(_LI('suspending %s'), six.text_type(self))
         return self._do_action(action)
 
     def resume(self):
@@ -740,15 +829,15 @@ class Resource(object):
         # Can't resume a resource unless it's SUSPEND_COMPLETE
         if self.state != (self.SUSPEND, self.COMPLETE):
             exc = exception.Error(_('State %s invalid for resume')
-                                  % str(self.state))
+                                  % six.text_type(self.state))
             raise exception.ResourceFailure(exc, self, action)
 
-        LOG.info(_('resuming %s') % str(self))
+        LOG.info(_LI('resuming %s'), six.text_type(self))
         return self._do_action(action)
 
     def snapshot(self):
         '''Snapshot the resource and return the created data, if any.'''
-        LOG.info(_('snapshotting %s') % str(self))
+        LOG.info(_LI('snapshotting %s'), six.text_type(self))
         return self._do_action(self.SNAPSHOT)
 
     @scheduler.wrappertask
@@ -756,12 +845,12 @@ class Resource(object):
         yield self.action_handler_task('delete_snapshot', args=[data])
 
     def physical_resource_name(self):
-        if self.id is None:
+        if self.id is None or self.action == self.INIT:
             return None
 
         name = '%s-%s-%s' % (self.stack.name,
                              self.name,
-                             short_id.get_id(self.id))
+                             short_id.get_id(self.uuid))
 
         if self.physical_resource_name_limit:
             name = self.reduce_physical_resource_name(
@@ -794,11 +883,22 @@ class Resource(object):
         return name[0:2] + '-' + name[-postfix_length:]
 
     def validate(self):
-        LOG.info(_('Validating %s') % str(self))
+        LOG.info(_LI('Validating %s'), six.text_type(self))
 
         function.validate(self.t)
         self.validate_deletion_policy(self.t.deletion_policy())
-        return self.properties.validate()
+        try:
+            validate = self.properties.validate(
+                with_value=self.stack.strict_validate)
+        except exception.StackValidationFailed as ex:
+            path = [self.stack.t.RESOURCES, ex.path[0],
+                    self.stack.t.get_section_name(ex.path[1])]
+            path.extend(ex.path[2:])
+            raise exception.StackValidationFailed(
+                error=ex.error,
+                path=path,
+                message=ex.error_message)
+        return validate
 
     @classmethod
     def validate_deletion_policy(cls, policy):
@@ -827,7 +927,7 @@ class Resource(object):
 
         initial_state = self.state
 
-        LOG.info(_('deleting %s') % str(self))
+        LOG.info(_LI('deleting %s'), six.text_type(self))
 
         with self._action_recorder(action):
             if self.abandon_in_progress:
@@ -853,7 +953,7 @@ class Resource(object):
             return
 
         try:
-            db_api.resource_get(self.context, self.id).delete()
+            resource_objects.Resource.delete(self.context, self.id)
         except exception.NotFound:
             # Don't fail on delete if the db entry has
             # not been created yet.
@@ -865,14 +965,13 @@ class Resource(object):
         self.resource_id = inst
         if self.id is not None:
             try:
-                rs = db_api.resource_get(self.context, self.id)
+                rs = resource_objects.Resource.get_obj(self.context, self.id)
                 rs.update_and_save({'nova_instance': self.resource_id})
             except Exception as ex:
-                LOG.warn(_('db error %s') % ex)
+                LOG.warn(_LW('db error %s'), ex)
 
-    def _store(self):
+    def _store(self, metadata=None):
         '''Create the resource in the database.'''
-        metadata = self.metadata_get()
         try:
             rs = {'action': self.action,
                   'status': self.status,
@@ -882,14 +981,20 @@ class Resource(object):
                   'name': self.name,
                   'rsrc_metadata': metadata,
                   'properties_data': self._stored_properties_data,
+                  'needed_by': self.needed_by,
+                  'requires': self.requires,
+                  'replaces': self.replaces,
+                  'replaced_by': self.replaced_by,
+                  'current_template_id': self.current_template_id,
                   'stack_name': self.stack.name}
 
-            new_rs = db_api.resource_create(self.context, rs)
+            new_rs = resource_objects.Resource.create(self.context, rs)
             self.id = new_rs.id
+            self.uuid = new_rs.uuid
             self.created_time = new_rs.created_at
             self._rsrc_metadata = metadata
         except Exception as ex:
-            LOG.error(_('DB error %s') % ex)
+            LOG.error(_LE('DB error %s'), ex)
 
     def _add_event(self, action, status, reason):
         '''Add a state change event to the database.'''
@@ -900,30 +1005,43 @@ class Resource(object):
         ev.store()
 
     def _store_or_update(self, action, status, reason):
+        prev_action = self.action
         self.action = action
         self.status = status
         self.status_reason = reason
 
+        data = {
+            'action': self.action,
+            'status': self.status,
+            'status_reason': reason,
+            'stack_id': self.stack.id,
+            'updated_at': self.updated_time,
+            'properties_data': self._stored_properties_data,
+            'needed_by': self.needed_by,
+            'requires': self.requires,
+            'replaces': self.replaces,
+            'replaced_by': self.replaced_by,
+            'current_template_id': self.current_template_id,
+            'nova_instance': self.resource_id
+        }
+        if prev_action == self.INIT:
+            metadata = self.t.metadata()
+            data['rsrc_metadata'] = metadata
+        else:
+            metadata = self._rsrc_metadata
+
         if self.id is not None:
             try:
-                rs = db_api.resource_get(self.context, self.id)
-                rs.update_and_save({
-                    'action': self.action,
-                    'status': self.status,
-                    'status_reason': reason,
-                    'stack_id': self.stack.id,
-                    'updated_at': self.updated_time,
-                    'properties_data': self._stored_properties_data,
-                    'nova_instance': self.resource_id})
+                rs = resource_objects.Resource.get_obj(self.context, self.id)
+                rs.update_and_save(data)
             except Exception as ex:
-                LOG.error(_('DB error %s') % ex)
-
-        # store resource in DB on transition to CREATE_IN_PROGRESS
-        # all other transitions (other than to DELETE_COMPLETE)
-        # should be handled by the update_and_save above..
-        elif (action, status) in [(self.CREATE, self.IN_PROGRESS),
-                                  (self.ADOPT, self.IN_PROGRESS)]:
-            self._store()
+                LOG.error(_LE('DB error %s'), ex)
+            else:
+                self._rsrc_metadata = metadata
+        else:
+            # This should only happen in unit tests
+            LOG.warning(_LW('Resource "%s" not pre-stored in DB'), self)
+            self._store(metadata)
 
     def _resolve_attribute(self, name):
         """
@@ -936,6 +1054,16 @@ class Resource(object):
         # By default, no attributes resolve
         pass
 
+    def regenerate_info_schema(self, definition):
+        """
+        Default implementation; should be overridden by resources that would
+        require schema refresh during update, ex. TemplateResource
+
+        :definition: Resource Definition
+        """
+        # By default, do not regenerate
+        pass
+
     def state_reset(self):
         """
         Reset state to (INIT, COMPLETE)
@@ -971,14 +1099,14 @@ class Resource(object):
         :results: the id or name of the resource.
         '''
         if self.resource_id is not None:
-            return unicode(self.resource_id)
+            return six.text_type(self.resource_id)
         else:
-            return unicode(self.name)
+            return six.text_type(self.name)
 
     def physical_resource_name_or_FnGetRefId(self):
         res_name = self.physical_resource_name()
         if res_name is not None:
-            return unicode(res_name)
+            return six.text_type(res_name)
         else:
             return Resource.FnGetRefId(self)
 
@@ -1013,10 +1141,16 @@ class Resource(object):
         to implement the signal, the base-class raise an exception if no
         handler is implemented.
         '''
+        if self.action in (self.SUSPEND, self.DELETE):
+            self._add_event(self.action, self.status,
+                            'Cannot signal resource during %s' % self.action)
+            ex = Exception(_('Cannot signal resource during %s') % self.action)
+            raise exception.ResourceFailure(ex, self)
+
         def get_string_details():
             if details is None:
                 return 'No signal details provided'
-            if isinstance(details, basestring):
+            if isinstance(details, six.string_types):
                 return details
             if isinstance(details, dict):
                 if all(k in details for k in ('previous', 'current',
@@ -1029,6 +1163,20 @@ class Resource(object):
                     return 'alarm state changed to %(state)s' % details
 
             return 'Unknown'
+
+        # Clear the hook without interfering with resources'
+        # `handle_signal` callbacks:
+        if (details and 'unset_hook' in details and
+                environment.valid_hook_type(details.get('unset_hook'))):
+            hook = details['unset_hook']
+            if self.has_hook(hook):
+                self.clear_hook(hook)
+                LOG.info(_LI('Clearing %(hook)s hook on %(resource)s'),
+                         {'hook': hook, 'resource': six.text_type(self)})
+                self._add_event(self.action, self.status,
+                                "Hook %s is cleared" % hook)
+                return
+
         if not callable(getattr(self, 'handle_signal', None)):
             raise exception.ResourceActionNotSupported(action='signal')
 
@@ -1038,10 +1186,10 @@ class Resource(object):
                 reason_string = "Signal: %s" % signal_result
             else:
                 reason_string = get_string_details()
-            self._add_event('signal', self.status, reason_string)
+            self._add_event('SIGNAL', self.status, reason_string)
         except Exception as ex:
-            LOG.exception(_('signal %(name)s : %(msg)s') % {'name': str(self),
-                                                            'msg': ex})
+            LOG.exception(_LE('signal %(name)s : %(msg)s')
+                          % {'name': six.text_type(self), 'msg': ex})
             failure = exception.ResourceFailure(ex, self)
             raise failure
 
@@ -1054,8 +1202,8 @@ class Resource(object):
         No-op for resources which don't explicitly override this method
         '''
         if new_metadata:
-            LOG.warning(_("Resource %s does not implement metadata update")
-                        % self.name)
+            LOG.warn(_LW("Resource %s does not implement metadata update"),
+                     self.name)
 
     @classmethod
     def resource_to_template(cls, resource_type):
@@ -1093,7 +1241,7 @@ class Resource(object):
         '''
         if self._data is None and self.id:
             try:
-                self._data = db_api.resource_data_get_all(self)
+                self._data = resource_data_objects.ResourceData.get_all(self)
             except exception.NotFound:
                 pass
 
@@ -1101,7 +1249,7 @@ class Resource(object):
 
     def data_set(self, key, value, redact=False):
         '''Save resource's key/value pair to database.'''
-        db_api.resource_data_set(self, key, value, redact)
+        resource_data_objects.ResourceData.set(self, key, value, redact)
         # force fetch all resource data from the database again
         self._data = None
 
@@ -1112,7 +1260,7 @@ class Resource(object):
         :returns: True if the key existed to delete
         '''
         try:
-            db_api.resource_data_delete(self, key)
+            resource_data_objects.ResourceData.delete(self, key)
         except exception.NotFound:
             return False
         else:
