@@ -17,23 +17,23 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
-cfg.CONF.import_opt('max_server_name_length', 'heat.common.config')
-
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LI
-from heat.common.i18n import _LW
 from heat.engine import attributes
-from heat.engine.clients import progress
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine import resource
-from heat.engine.resources import scheduler_hints as sh
+from heat.engine import scheduler
+from heat.engine import volume_tasks as vol_task
+
+cfg.CONF.import_opt('instance_user', 'heat.common.config')
+cfg.CONF.import_opt('stack_scheduler_hints', 'heat.common.config')
 
 LOG = logging.getLogger(__name__)
 
 
-class Instance(resource.Resource, sh.SchedulerHintsMixin):
+class Instance(resource.Resource):
 
     PROPERTIES = (
         IMAGE_ID, INSTANCE_TYPE, KEY_NAME, AVAILABILITY_ZONE,
@@ -322,28 +322,25 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
     attributes_schema = {
         AVAILABILITY_ZONE_ATTR: attributes.Schema(
             _('The Availability Zone where the specified instance is '
-              'launched.'),
-            type=attributes.Schema.STRING
+              'launched.')
         ),
         PRIVATE_DNS_NAME: attributes.Schema(
-            _('Private DNS name of the specified instance.'),
-            type=attributes.Schema.STRING
+            _('Private DNS name of the specified instance.')
         ),
         PUBLIC_DNS_NAME: attributes.Schema(
-            _('Public DNS name of the specified instance.'),
-            type=attributes.Schema.STRING
+            _('Public DNS name of the specified instance.')
         ),
         PRIVATE_IP: attributes.Schema(
-            _('Private IP address of the specified instance.'),
-            type=attributes.Schema.STRING
+            _('Private IP address of the specified instance.')
         ),
         PUBLIC_IP: attributes.Schema(
-            _('Public IP address of the specified instance.'),
-            type=attributes.Schema.STRING
+            _('Public IP address of the specified instance.')
         ),
     }
 
-    physical_resource_name_limit = cfg.CONF.max_server_name_length
+    # Server host name limit to 53 characters by due to typical default
+    # linux HOST_NAME_MAX of 64, minus the .novalocal appended to the name
+    physical_resource_name_limit = 53
 
     default_client_name = 'nova'
 
@@ -352,21 +349,19 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         self.ipaddress = None
 
     def _set_ipaddress(self, networks):
-        """Set IP address to self.ipaddress from a list of networks.
-
-        Read the server's IP address from a list of networks provided by Nova.
-        """
+        '''
+        Read the server's IP address from a list of networks provided by Nova
+        '''
         # Just record the first ipaddress
-        for n in sorted(networks, reverse=True):
+        for n in networks:
             if len(networks[n]) > 0:
                 self.ipaddress = networks[n][0]
                 break
 
     def _ipaddress(self):
-        """Return the server's IP address.
-
-        Fetching it from Nova if necessary.
-        """
+        '''
+        Return the server's IP address, fetching it from Nova if necessary
+        '''
         if self.ipaddress is None:
             self.ipaddress = self.client_plugin().server_to_ipaddress(
                 self.resource_id)
@@ -374,20 +369,20 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return self.ipaddress or '0.0.0.0'
 
     def _availability_zone(self):
-        """Return Server's Availability Zone.
-
-        Fetching it from Nova if necessary.
-        """
+        '''
+        Return Server's Availability Zone, fetching it from Nova if necessary.
+        '''
         availability_zone = self.properties[self.AVAILABILITY_ZONE]
         if availability_zone is None:
             try:
-                server = self.client().servers.get(self.resource_id)
+                server = self.nova().servers.get(self.resource_id)
+                availability_zone = getattr(server,
+                                            'OS-EXT-AZ:availability_zone')
             except Exception as e:
                 self.client_plugin().ignore_not_found(e)
                 return
-        # Default to None if Nova's
-        # OS-EXT-AZ:availability_zone extension is disabled
-        return getattr(server, 'OS-EXT-AZ:availability_zone', None)
+
+        return availability_zone
 
     def _resolve_attribute(self, name):
         res = None
@@ -404,8 +399,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         # delete the port data which implicit-created
         port_id = self.data().get('port_id')
         if port_id:
-            with self.client_plugin('neutron').ignore_not_found:
+            try:
                 self.neutron().delete_port(port_id)
+            except Exception as ex:
+                self.client_plugin('neutron').ignore_not_found(ex)
             self.data_delete('port_id')
 
     def _build_nics(self, network_interfaces,
@@ -510,10 +507,9 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
 
         image_name = self.properties[self.IMAGE_ID]
 
-        image_id = self.client_plugin(
-            'glance').find_image_by_name_or_id(image_name)
+        image_id = self.client_plugin('glance').get_image_id(image_name)
 
-        flavor_id = self.client_plugin().find_flavor_by_name_or_id(flavor)
+        flavor_id = self.client_plugin().get_flavor_id(flavor)
 
         scheduler_hints = {}
         if self.properties[self.NOVA_SCHEDULER_HINTS]:
@@ -529,7 +525,14 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
                     scheduler_hints[hint] = hint_value
         else:
             scheduler_hints = None
-        scheduler_hints = self._scheduler_hints(scheduler_hints)
+        if cfg.CONF.stack_scheduler_hints:
+            if scheduler_hints is None:
+                scheduler_hints = {}
+            scheduler_hints['heat_root_stack_id'] = self.stack.root_stack_id()
+            scheduler_hints['heat_stack_id'] = self.stack.id
+            scheduler_hints['heat_stack_name'] = self.stack.name
+            scheduler_hints['heat_path_in_stack'] = self.stack.path_in_stack()
+            scheduler_hints['heat_resource_name'] = self.name
 
         nics = self._build_nics(self.properties[self.NETWORK_INTERFACES],
                                 security_groups=security_groups,
@@ -540,15 +543,23 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
 
         server = None
 
+        # FIXME(shadower): the instance_user config option is deprecated. Once
+        # it's gone, we should always use ec2-user for compatibility with
+        # CloudFormation.
+        if cfg.CONF.instance_user:
+            instance_user = cfg.CONF.instance_user
+        else:
+            instance_user = 'ec2-user'
+
         try:
-            server = self.client().servers.create(
+            server = self.nova().servers.create(
                 name=self.physical_resource_name(),
                 image=image_id,
                 flavor=flavor_id,
                 key_name=self.properties[self.KEY_NAME],
                 security_groups=security_groups,
                 userdata=self.client_plugin().build_userdata(
-                    self.metadata_get(), userdata, 'ec2-user'),
+                    self.metadata_get(), userdata, instance_user),
                 meta=self._get_nova_metadata(self.properties),
                 scheduler_hints=scheduler_hints,
                 nics=nics,
@@ -560,52 +571,59 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
             if server is not None:
                 self.resource_id_set(server.id)
 
-        creator = progress.ServerCreateProgress(server.id)
-        attachers = []
-        for vol_id, device in self.volumes():
-            attachers.append(progress.VolumeAttachProgress(self.resource_id,
-                                                           vol_id, device))
-        return creator, tuple(attachers)
+        return server, scheduler.TaskRunner(self._attach_volumes_task())
+
+    def _attach_volumes_task(self):
+        attach_tasks = (vol_task.VolumeAttachTask(self.stack,
+                                                  self.resource_id,
+                                                  volume_id,
+                                                  device)
+                        for volume_id, device in self.volumes())
+        return scheduler.PollingTaskGroup(attach_tasks)
 
     def check_create_complete(self, cookie):
-        creator, attachers = cookie
+        server, volume_attach_task = cookie
+        return (self._check_active(server) and
+                self._check_volume_attached(server, volume_attach_task))
 
-        if not creator.complete:
-            creator.complete = self.client_plugin()._check_active(
-                creator.server_id, 'Instance')
-            if creator.complete:
-                server = self.client_plugin().get_server(creator.server_id)
-                self._set_ipaddress(server.networks)
-                # NOTE(pas-ha) small optimization,
-                # return True if there are no volumes to attach
-                # to save one check_create_complete call
-                return not len(attachers)
-            else:
-                return False
-        return self._attach_volumes(attachers)
+    def _check_volume_attached(self, server, volume_attach_task):
+        if not volume_attach_task.started():
+            self._set_ipaddress(server.networks)
+            volume_attach_task.start()
+            return volume_attach_task.done()
+        else:
+            return volume_attach_task.step()
 
-    def _attach_volumes(self, attachers):
-        for attacher in attachers:
-            if not attacher.called:
-                self.client_plugin().attach_volume(attacher.srv_id,
-                                                   attacher.vol_id,
-                                                   attacher.device)
-                attacher.called = True
-                return False
+    def _check_active(self, server):
+        cp = self.client_plugin()
+        status = cp.get_status(server)
+        if status != 'ACTIVE':
+            cp.refresh_server(server)
+            status = cp.get_status(server)
 
-        for attacher in attachers:
-            if not attacher.complete:
-                attacher.complete = self.client_plugin(
-                    'cinder').check_attach_volume_complete(attacher.vol_id)
-                break
-        out = all(attacher.complete for attacher in attachers)
-        return out
+        if status == 'ACTIVE':
+            return True
+
+        if status in cp.deferred_server_statuses:
+            return False
+
+        if status == 'ERROR':
+            fault = getattr(server, 'fault', {})
+            raise resource.ResourceInError(
+                resource_status=status,
+                status_reason=_("Message: %(message)s, Code: %(code)s") % {
+                    'message': fault.get('message', _('Unknown')),
+                    'code': fault.get('code', _('Unknown'))
+                })
+
+        raise resource.ResourceUnknownStatus(
+            resource_status=server.status,
+            result=_('Instance is not active'))
 
     def volumes(self):
-        """Return an iterator for all volumes that should be attached.
-
-        Return an iterator over (volume_id, device) tuples for all volumes that
-        should be attached to this instance.
+        """
+        Return an iterator over (volume_id, device) tuples for all volumes
+        that should be attached to this instance.
         """
         volumes = self.properties[self.VOLUMES]
 
@@ -621,166 +639,125 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
                 old_network_ifaces.remove(iface)
 
     def handle_check(self):
-        server = self.client().servers.get(self.resource_id)
-        if not self.client_plugin()._check_active(server, 'Instance'):
+        server = self.nova().servers.get(self.resource_id)
+        if not self._check_active(server):
             raise exception.Error(_("Instance is not ACTIVE (was: %s)") %
                                   server.status.strip())
 
-    def _update_instance_type(self, prop_diff):
-        flavor = prop_diff[self.INSTANCE_TYPE]
-        flavor_id = self.client_plugin().find_flavor_by_name_or_id(flavor)
-        handler_args = {'args': (flavor_id,)}
-        checker_args = {'args': (flavor_id,)}
-
-        prg_resize = progress.ServerUpdateProgress(self.resource_id,
-                                                   'resize',
-                                                   handler_extra=handler_args,
-                                                   checker_extra=checker_args)
-        prg_verify = progress.ServerUpdateProgress(self.resource_id,
-                                                   'verify_resize')
-        return prg_resize, prg_verify
-
-    def _update_network_interfaces(self, server, prop_diff):
-        updaters = []
-        new_network_ifaces = prop_diff.get(self.NETWORK_INTERFACES)
-        old_network_ifaces = self.properties.get(self.NETWORK_INTERFACES)
-
-        # if there is entrys in old_network_ifaces and new_network_ifaces,
-        # remove the same entrys from old and new ifaces
-        if old_network_ifaces and new_network_ifaces:
-            # there are four situations:
-            # 1.old includes new, such as: old = 2,3, new = 2
-            # 2.new includes old, such as: old = 2,3, new = 1,2,3
-            # 3.has overlaps, such as: old = 2,3, new = 1,2
-            # 4.different, such as: old = 2,3, new = 1,4
-            # detach unmatched ones in old, attach unmatched ones in new
-            self._remove_matched_ifaces(old_network_ifaces,
-                                        new_network_ifaces)
-            if old_network_ifaces:
-                old_nics = self._build_nics(old_network_ifaces)
-                for nic in old_nics:
-                    updaters.append(
-                        progress.ServerUpdateProgress(
-                            self.resource_id, 'interface_detach',
-                            complete=True,
-                            handler_extra={'args': (nic['port-id'],)})
-                    )
-            if new_network_ifaces:
-                new_nics = self._build_nics(new_network_ifaces)
-                for nic in new_nics:
-                    handler_kwargs = {'port_id': nic['port-id']}
-                    updaters.append(
-                        progress.ServerUpdateProgress(
-                            self.resource_id, 'interface_attach',
-                            complete=True,
-                            handler_extra={'kwargs': handler_kwargs})
-                    )
-        # if there is no change of 'NetworkInterfaces', do nothing,
-        # keep the behavior as creation
-        elif (old_network_ifaces and
-                (self.NETWORK_INTERFACES not in prop_diff)):
-            LOG.warning(_LW('There is no change of "%(net_interfaces)s" '
-                            'for instance %(server)s, do nothing '
-                            'when updating.'),
-                        {'net_interfaces': self.NETWORK_INTERFACES,
-                         'server': self.resource_id})
-        # if the interfaces not come from property 'NetworkInterfaces',
-        # the situation is somewhat complex, so to detach the old ifaces,
-        # and then attach the new ones.
-        else:
-            subnet_id = (prop_diff.get(self.SUBNET_ID) or
-                         self.properties.get(self.SUBNET_ID))
-            security_groups = self._get_security_groups()
-            if not server:
-                server = self.client().servers.get(self.resource_id)
-
-            interfaces = server.interface_list()
-            for iface in interfaces:
-                updaters.append(
-                    progress.ServerUpdateProgress(
-                        self.resource_id, 'interface_detach',
-                        complete=True,
-                        handler_extra={'args': (iface.port_id,)})
-                )
-            # first to delete the port which implicit-created by heat
-            self._port_data_delete()
-            nics = self._build_nics(new_network_ifaces,
-                                    security_groups=security_groups,
-                                    subnet_id=subnet_id)
-            # 'SubnetId' property is empty(or None) and
-            # 'NetworkInterfaces' property is empty(or None),
-            # _build_nics() will return nics = None,we should attach
-            # first free port, according to similar behavior during
-            # instance creation
-            if not nics:
-                updaters.append(
-                    progress.ServerUpdateProgress(
-                        self.resource_id, 'interface_attach', complete=True)
-                )
-            else:
-                for nic in nics:
-                    updaters.append(
-                        progress.ServerUpdateProgress(
-                            self.resource_id, 'interface_attach',
-                            complete=True,
-                            handler_extra={'kwargs':
-                                           {'port_id': nic['port-id']}})
-                    )
-
-        return updaters
-
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        if tmpl_diff.metadata_changed():
-            self.metadata_set(json_snippet.metadata())
-        updaters = []
+        if 'Metadata' in tmpl_diff:
+            self.metadata_set(tmpl_diff['Metadata'])
+        checkers = []
         server = None
         if self.TAGS in prop_diff:
-            server = self.client().servers.get(self.resource_id)
+            server = self.nova().servers.get(self.resource_id)
             self.client_plugin().meta_update(
                 server, self._get_nova_metadata(prop_diff))
 
         if self.INSTANCE_TYPE in prop_diff:
-            updaters.extend(self._update_instance_type(prop_diff))
+            flavor = prop_diff[self.INSTANCE_TYPE]
+            flavor_id = self.client_plugin().get_flavor_id(flavor)
+            if not server:
+                server = self.nova().servers.get(self.resource_id)
+            checker = scheduler.TaskRunner(self.client_plugin().resize,
+                                           server, flavor, flavor_id)
+            checkers.append(checker)
+        if self.NETWORK_INTERFACES in prop_diff:
+            new_network_ifaces = prop_diff.get(self.NETWORK_INTERFACES)
+            old_network_ifaces = self.properties.get(self.NETWORK_INTERFACES)
+            subnet_id = (
+                prop_diff.get(self.SUBNET_ID) or
+                self.properties.get(self.SUBNET_ID))
+            security_groups = self._get_security_groups()
+            if not server:
+                server = self.nova().servers.get(self.resource_id)
+            # if there is entrys in old_network_ifaces and new_network_ifaces,
+            # remove the same entrys from old and new ifaces
+            if old_network_ifaces and new_network_ifaces:
+                # there are four situations:
+                # 1.old includes new, such as: old = 2,3, new = 2
+                # 2.new includes old, such as: old = 2,3, new = 1,2,3
+                # 3.has overlaps, such as: old = 2,3, new = 1,2
+                # 4.different, such as: old = 2,3, new = 1,4
+                # detach unmatched ones in old, attach unmatched ones in new
+                self._remove_matched_ifaces(old_network_ifaces,
+                                            new_network_ifaces)
+                if old_network_ifaces:
+                    old_nics = self._build_nics(old_network_ifaces)
+                    for nic in old_nics:
+                        checker = scheduler.TaskRunner(
+                            server.interface_detach,
+                            nic['port-id'])
+                        checkers.append(checker)
+                if new_network_ifaces:
+                    new_nics = self._build_nics(new_network_ifaces)
+                    for nic in new_nics:
+                        checker = scheduler.TaskRunner(
+                            server.interface_attach,
+                            nic['port-id'],
+                            None, None)
+                        checkers.append(checker)
+            # if the interfaces not come from property 'NetworkInterfaces',
+            # the situation is somewhat complex, so to detach the old ifaces,
+            # and then attach the new ones.
+            else:
+                interfaces = server.interface_list()
+                for iface in interfaces:
+                    checker = scheduler.TaskRunner(server.interface_detach,
+                                                   iface.port_id)
+                    checkers.append(checker)
+                # first to delete the port which implicit-created by heat
+                self._port_data_delete()
+                nics = self._build_nics(new_network_ifaces,
+                                        security_groups=security_groups,
+                                        subnet_id=subnet_id)
+                # 'SubnetId' property is empty(or None) and
+                # 'NetworkInterfaces' property is empty(or None),
+                # _build_nics() will return nics = None,we should attach
+                # first free port, according to similar behavior during
+                # instance creation
+                if not nics:
+                    checker = scheduler.TaskRunner(server.interface_attach,
+                                                   None, None, None)
+                    checkers.append(checker)
+                else:
+                    for nic in nics:
+                        checker = scheduler.TaskRunner(
+                            server.interface_attach,
+                            nic['port-id'], None, None)
+                        checkers.append(checker)
 
-        if (self.NETWORK_INTERFACES in prop_diff or
-                self.SUBNET_ID in prop_diff):
-            updaters.extend(self._update_network_interfaces(server, prop_diff))
+        if checkers:
+            checkers[0].start()
+        return checkers
 
-        # NOTE(pas-ha) optimization is possible (starting first task
-        # right away), but we'd rather not, as this method already might
-        # have called several APIs
-        return updaters
-
-    def check_update_complete(self, updaters):
-        """Push all updaters to completion in list order."""
-        for prg in updaters:
-            if not prg.called:
-                handler = getattr(self.client_plugin(), prg.handler)
-                prg.called = handler(*prg.handler_args,
-                                     **prg.handler_kwargs)
+    def check_update_complete(self, checkers):
+        '''Push all checkers to completion in list order.'''
+        for checker in checkers:
+            if not checker.started():
+                checker.start()
+            if not checker.step():
                 return False
-            if not prg.complete:
-                check_complete = getattr(self.client_plugin(), prg.checker)
-                prg.complete = check_complete(*prg.checker_args,
-                                              **prg.checker_kwargs)
-                break
-        return all(prg.complete for prg in updaters)
+        return True
 
     def metadata_update(self, new_metadata=None):
-        """Refresh the metadata if new_metadata is None."""
+        '''
+        Refresh the metadata if new_metadata is None
+        '''
         if new_metadata is None:
             self.metadata_set(self.t.metadata())
 
     def validate(self):
-        """Validate any of the provided params."""
+        '''
+        Validate any of the provided params
+        '''
         res = super(Instance, self).validate()
         if res:
             return res
 
         # check validity of security groups vs. network interfaces
         security_groups = self._get_security_groups()
-        network_interfaces = self.properties.get(self.NETWORK_INTERFACES)
-        if security_groups and network_interfaces:
+        if security_groups and self.properties.get(self.NETWORK_INTERFACES):
             raise exception.ResourcePropertyConflict(
                 '/'.join([self.SECURITY_GROUPS, self.SECURITY_GROUP_IDS]),
                 self.NETWORK_INTERFACES)
@@ -802,16 +779,15 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
                             "when specifying BlockDeviceMappings.")
                     raise exception.StackValidationFailed(message=msg)
 
-        subnet_id = self.properties.get(self.SUBNET_ID)
-        if network_interfaces and subnet_id:
-            # consider the old templates, we only to log to warn user
-            # NetworkInterfaces has higher priority than SubnetId
-            LOG.warning(_LW('"%(subnet)s" will be ignored if specified '
-                            '"%(net_interfaces)s". So if you specified the '
-                            '"%(net_interfaces)s" property, '
-                            'do not specify "%(subnet)s" property.'),
-                        {'subnet': self.SUBNET_ID,
-                         'net_interfaces': self.NETWORK_INTERFACES})
+    def _detach_volumes_task(self):
+        '''
+        Detach volumes from the instance
+        '''
+        detach_tasks = (vol_task.VolumeDetachTask(self.stack,
+                                                  self.resource_id,
+                                                  volume_id)
+                        for volume_id, device in self.volumes())
+        return scheduler.PollingTaskGroup(detach_tasks)
 
     def handle_delete(self):
         # make sure to delete the port which implicit-created by heat
@@ -820,89 +796,109 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         if self.resource_id is None:
             return
         try:
-            self.client().servers.delete(self.resource_id)
+            server = self.nova().servers.get(self.resource_id)
         except Exception as e:
             self.client_plugin().ignore_not_found(e)
             return
-        return self.resource_id
+        deleters = (
+            scheduler.TaskRunner(self._detach_volumes_task()),
+            scheduler.TaskRunner(self.client_plugin().delete_server,
+                                 server))
+        deleters[0].start()
+        return deleters
 
-    def check_delete_complete(self, server_id):
-        if not server_id:
-            return True
-        return self.client_plugin().check_delete_server_complete(server_id)
+    def check_delete_complete(self, deleters):
+        # if the resource was already deleted, deleters will be None
+        if deleters:
+            for deleter in deleters:
+                if not deleter.started():
+                    deleter.start()
+                if not deleter.step():
+                    return False
+        return True
 
     def handle_suspend(self):
-        """Suspend an instance.
-
-        Note we do not wait for the SUSPENDED state, this is polled for by
-        check_suspend_complete in a similar way to the create logic so we can
-        take advantage of coroutines.
-        """
+        '''
+        Suspend an instance - note we do not wait for the SUSPENDED state,
+        this is polled for by check_suspend_complete in a similar way to the
+        create logic so we can take advantage of coroutines
+        '''
         if self.resource_id is None:
             raise exception.Error(_('Cannot suspend %s, resource_id not set') %
                                   self.name)
 
         try:
-            server = self.client().servers.get(self.resource_id)
+            server = self.nova().servers.get(self.resource_id)
         except Exception as e:
             if self.client_plugin().is_not_found(e):
                 raise exception.NotFound(_('Failed to find instance %s') %
                                          self.resource_id)
-            else:
-                raise
         else:
-            # if the instance has been suspended successful,
-            # no need to suspend again
-            if self.client_plugin().get_status(server) != 'SUSPENDED':
-                LOG.debug("suspending instance %s" % self.resource_id)
-                server.suspend()
-            return server.id
+            LOG.debug("suspending instance %s" % self.resource_id)
+            # We want the server.suspend to happen after the volume
+            # detachement has finished, so pass both tasks and the server
+            suspend_runner = scheduler.TaskRunner(server.suspend)
+            volumes_runner = scheduler.TaskRunner(self._detach_volumes_task())
+            return server, suspend_runner, volumes_runner
 
-    def check_suspend_complete(self, server_id):
-        cp = self.client_plugin()
-        server = cp.fetch_server(server_id)
-        if not server:
-            return False
-        status = cp.get_status(server)
-        LOG.debug('%(name)s check_suspend_complete status = %(status)s'
-                  % {'name': self.name, 'status': status})
-        if status in list(cp.deferred_server_statuses + ['ACTIVE']):
-            return status == 'SUSPENDED'
+    def check_suspend_complete(self, cookie):
+        server, suspend_runner, volumes_runner = cookie
+
+        if not volumes_runner.started():
+            volumes_runner.start()
+
+        if volumes_runner.done():
+            if not suspend_runner.started():
+                suspend_runner.start()
+
+            if suspend_runner.done():
+                if server.status == 'SUSPENDED':
+                    return True
+
+                cp = self.client_plugin()
+                cp.refresh_server(server)
+                LOG.debug("%(name)s check_suspend_complete "
+                          "status = %(status)s",
+                          {'name': self.name, 'status': server.status})
+                if server.status in list(cp.deferred_server_statuses +
+                                         ['ACTIVE']):
+                    return server.status == 'SUSPENDED'
+                else:
+                    raise exception.Error(_(' nova reported unexpected '
+                                            'instance[%(instance)s] '
+                                            'status[%(status)s]') %
+                                          {'instance': self.name,
+                                           'status': server.status})
+            else:
+                suspend_runner.step()
         else:
-            exc = exception.ResourceUnknownStatus(
-                result=_('Suspend of instance %s failed') % server.name,
-                resource_status=status)
-            raise exc
+            volumes_runner.step()
 
     def handle_resume(self):
-        """Resume an instance.
-
-        Note we do not wait for the ACTIVE state, this is polled for by
-        check_resume_complete in a similar way to the create logic so we can
-        take advantage of coroutines.
-        """
+        '''
+        Resume an instance - note we do not wait for the ACTIVE state,
+        this is polled for by check_resume_complete in a similar way to the
+        create logic so we can take advantage of coroutines
+        '''
         if self.resource_id is None:
             raise exception.Error(_('Cannot resume %s, resource_id not set') %
                                   self.name)
 
         try:
-            server = self.client().servers.get(self.resource_id)
+            server = self.nova().servers.get(self.resource_id)
         except Exception as e:
             if self.client_plugin().is_not_found(e):
                 raise exception.NotFound(_('Failed to find instance %s') %
                                          self.resource_id)
-            else:
-                raise
         else:
-            # if the instance has been resumed successful,
-            # no need to resume again
-            if self.client_plugin().get_status(server) != 'ACTIVE':
-                LOG.debug("resuming instance %s" % self.resource_id)
-                server.resume()
-            return server.id
+            LOG.debug("resuming instance %s" % self.resource_id)
+            server.resume()
+            return server, scheduler.TaskRunner(self._attach_volumes_task())
 
-    def check_resume_complete(self, server_id):
-        return self.client_plugin()._check_active(server_id, 'Instance')
+    def check_resume_complete(self, cookie):
+        server, volume_attach_task = cookie
+        return (self._check_active(server) and
+                self._check_volume_attached(server, volume_attach_task))
 
 
 def resource_mapping():
