@@ -11,34 +11,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
-
+from oslo_serialization import jsonutils
 from requests import exceptions
 import six
 
 from heat.common import exception
+from heat.common.i18n import _
 from heat.common import template_format
 from heat.common import urlfetch
 from heat.engine import attributes
 from heat.engine import environment
 from heat.engine import properties
-from heat.engine import stack_resource
+from heat.engine.resources import stack_resource
 from heat.engine import template
 
 
-def generate_class(name, template_name):
-    try:
-        data = urlfetch.get(template_name, allowed_schemes=('file',))
-    except IOError:
-        return TemplateResource
+def generate_class(name, template_name, env):
+    data = TemplateResource.get_template_file(template_name, ('file',))
     tmpl = template.Template(template_format.parse(data))
-    properties_schema = properties.Properties.schema_from_params(
-        tmpl.param_schemata())
-    attributes_schema = attributes.Attributes.schema_from_outputs(
-        tmpl[tmpl.OUTPUTS])
+    props, attrs = TemplateResource.get_schemas(tmpl, env.param_defaults)
     cls = type(name, (TemplateResource,),
-               {"properties_schema": properties_schema,
-                "attributes_schema": attributes_schema})
+               {'properties_schema': props,
+                'attributes_schema': attrs})
     return cls
 
 
@@ -56,8 +50,21 @@ class TemplateResource(stack_resource.StackResource):
         self.stack = stack
         self.validation_exception = None
 
-        tri = stack.env.get_resource_info(
-            json_snippet['Type'],
+        tri = self._get_resource_info(json_snippet)
+
+        self.properties_schema = {}
+        self.attributes_schema = {}
+
+        # run Resource.__init__() so we can call self.nested()
+        super(TemplateResource, self).__init__(name, json_snippet, stack)
+        self.resource_info = tri
+        if self.validation_exception is None:
+            self._generate_schema(self.t)
+
+    def _get_resource_info(self, rsrc_defn):
+        tri = self.stack.env.get_resource_info(
+            rsrc_defn.resource_type,
+            resource_name=rsrc_defn.name,
             registry_type=environment.TemplateResourceInfo)
         if tri is None:
             self.validation_exception = ValueError(_(
@@ -65,32 +72,43 @@ class TemplateResource(stack_resource.StackResource):
                 '.template are supported'))
         else:
             self.template_name = tri.template_name
+            self.resource_type = tri.name
             if tri.user_resource:
                 self.allowed_schemes = ('http', 'https')
             else:
                 self.allowed_schemes = ('http', 'https', 'file')
 
-        # run Resource.__init__() so we can call self.nested()
-        self.properties_schema = {}
-        self.attributes_schema = {}
-        super(TemplateResource, self).__init__(name, json_snippet, stack)
-        if self.validation_exception is None:
-            self._generate_schema(self.t)
+        return tri
+
+    @staticmethod
+    def get_template_file(template_name, allowed_schemes):
+        try:
+            return urlfetch.get(template_name, allowed_schemes=allowed_schemes)
+        except (IOError, exceptions.RequestException) as r_exc:
+            args = {'name': template_name, 'exc': six.text_type(r_exc)}
+            msg = _('Could not fetch remote template '
+                    '"%(name)s": %(exc)s') % args
+            raise exception.NotFound(msg_fmt=msg)
+
+    @staticmethod
+    def get_schemas(tmpl, param_defaults):
+        return ((properties.Properties.schema_from_params(
+                tmpl.param_schemata(param_defaults))),
+                (attributes.Attributes.schema_from_outputs(
+                 tmpl[tmpl.OUTPUTS])))
 
     def _generate_schema(self, definition):
         self._parsed_nested = None
         try:
             tmpl = template.Template(self.child_template())
-        except ValueError as download_error:
+        except (exception.NotFound, ValueError) as download_error:
             self.validation_exception = download_error
             tmpl = template.Template(
                 {"HeatTemplateFormatVersion": "2012-12-12"})
 
         # re-generate the properties and attributes from the template.
-        self.properties_schema = (properties.Properties
-                                  .schema_from_params(tmpl.param_schemata()))
-        self.attributes_schema = (attributes.Attributes
-                                  .schema_from_outputs(tmpl[tmpl.OUTPUTS]))
+        self.properties_schema, self.attributes_schema = self.get_schemas(
+            tmpl, self.stack.env.param_defaults)
 
         self.properties = definition.properties(self.properties_schema,
                                                 self.context)
@@ -143,6 +161,10 @@ class TemplateResource(stack_resource.StackResource):
             self._parsed_nested = template_format.parse(self.template_data())
         return self._parsed_nested
 
+    def regenerate_info_schema(self, definition):
+        self._get_resource_info(definition)
+        self._generate_schema(definition)
+
     def implementation_signature(self):
         self._generate_schema(self.t)
         return super(TemplateResource, self).implementation_signature()
@@ -156,20 +178,21 @@ class TemplateResource(stack_resource.StackResource):
         t_data = self.stack.t.files.get(self.template_name)
         if not t_data and self.template_name.endswith((".yaml", ".template")):
             try:
-                t_data = urlfetch.get(self.template_name,
-                                      allowed_schemes=self.allowed_schemes)
-            except (exceptions.RequestException, IOError) as r_exc:
-                reported_excp = ValueError(_("Could not fetch remote template "
-                                             "'%(name)s': %(exc)s") % {
-                                                 'name': self.template_name,
-                                                 'exc': str(r_exc)})
+                t_data = self.get_template_file(self.template_name,
+                                                self.allowed_schemes)
+            except exception.NotFound as err:
+                if self.action == self.UPDATE:
+                    raise
+                reported_excp = err
 
         if t_data is None:
             if self.nested() is not None:
-                t_data = json.dumps(self.nested().t.t)
+                t_data = jsonutils.dumps(self.nested().t.t)
 
         if t_data is not None:
             self.stack.t.files[self.template_name] = t_data
+            self.stack.t.env.register_class(self.resource_type,
+                                            self.template_name)
             return t_data
         if reported_excp is None:
             reported_excp = ValueError(_('Unknown error retrieving %s') %
@@ -222,6 +245,7 @@ class TemplateResource(stack_resource.StackResource):
             raise exception.StackValidationFailed(message=msg)
         cri = self.stack.env.get_resource_info(
             self.type(),
+            resource_name=self.name,
             registry_type=environment.ClassResourceInfo)
 
         # If we're using an existing resource type as a facade for this
@@ -237,21 +261,18 @@ class TemplateResource(stack_resource.StackResource):
                                          self.child_params(),
                                          adopt_data=resource_data)
 
-    def check_adopt_complete(self, stack_creator):
-        done = stack_creator.step()
-        if done:
-            if self._nested.state != (self._nested.ADOPT,
-                                      self._nested.COMPLETE):
-                raise exception.Error(self._nested.status_reason)
-
-        return done
-
     def handle_create(self):
         return self.create_with_template(self.child_template(),
                                          self.child_params())
 
+    def metadata_update(self, new_metadata=None):
+        '''
+        Refresh the metadata if new_metadata is None
+        '''
+        if new_metadata is None:
+            self.metadata_set(self.t.metadata())
+
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        self._generate_schema(json_snippet)
         return self.update_with_template(self.child_template(),
                                          self.child_params())
 
@@ -259,6 +280,48 @@ class TemplateResource(stack_resource.StackResource):
         return self.delete_nested()
 
     def FnGetRefId(self):
-        if not self.nested():
-            return unicode(self.name)
+        if self.nested() is None:
+            return six.text_type(self.name)
+
+        if 'OS::stack_id' in self.nested().outputs:
+            return self.nested().output('OS::stack_id')
+
         return self.nested().identifier().arn()
+
+    def FnGetAtt(self, key, *path):
+        stack = self.nested()
+        if stack is None:
+            return None
+
+        def _get_inner_resource(resource_name):
+            if self.nested() is not None:
+                try:
+                    return self.nested()[resource_name]
+                except KeyError:
+                    raise exception.ResourceNotFound(
+                        resource_name=resource_name,
+                        stack_name=self.nested().name)
+
+        def get_rsrc_attr(resource_name, *attr_path):
+            resource = _get_inner_resource(resource_name)
+            return resource.FnGetAtt(*attr_path)
+
+        def get_rsrc_id(resource_name):
+            resource = _get_inner_resource(resource_name)
+            return resource.FnGetRefId()
+
+        # first look for explicit resource.x.y
+        if key.startswith('resource.'):
+            npath = key.split(".", 2)[1:] + list(path)
+            if len(npath) > 1:
+                return get_rsrc_attr(*npath)
+            else:
+                return get_rsrc_id(*npath)
+
+        # then look for normal outputs
+        if key in stack.outputs:
+            return attributes.select_from_attribute(stack.output(key), path)
+
+        # otherwise the key must be wrong.
+        raise exception.InvalidTemplateAttribute(resource=self.name,
+                                                 key=key)
