@@ -15,8 +15,9 @@ import copy
 import itertools
 import operator
 
-from heat.common import exception
+import six
 
+from heat.common import exception
 from heat.engine import function
 from heat.engine import properties
 
@@ -37,7 +38,7 @@ class ResourceDefinitionCore(object):
 
     def __init__(self, name, resource_type, properties=None, metadata=None,
                  depends=None, deletion_policy=None, update_policy=None,
-                 description=''):
+                 description=None):
         """
         Initialise with the parsed definition of a resource.
 
@@ -53,9 +54,10 @@ class ResourceDefinitionCore(object):
         :param update_policy: A dictionary of supplied update policies
         :param description: A string describing the resource
         """
+        depends = depends or []
         self.name = name
         self.resource_type = resource_type
-        self.description = description
+        self.description = description or ''
         self._properties = properties
         self._metadata = metadata
         self._depends = depends
@@ -65,7 +67,7 @@ class ResourceDefinitionCore(object):
         self._hash = hash(self.resource_type)
         self._rendering = None
 
-        assert isinstance(description, basestring)
+        assert isinstance(self.description, six.string_types)
 
         if properties is not None:
             assert isinstance(properties, (collections.Mapping,
@@ -77,11 +79,10 @@ class ResourceDefinitionCore(object):
                                          function.Function))
             self._hash ^= _hash_data(metadata)
 
-        if depends is not None:
-            assert isinstance(depends, (collections.Sequence,
-                                        function.Function))
-            assert not isinstance(depends, basestring)
-            self._hash ^= _hash_data(depends)
+        assert isinstance(depends, (collections.Sequence,
+                                    function.Function))
+        assert not isinstance(depends, six.string_types)
+        self._hash ^= _hash_data(depends)
 
         if deletion_policy is not None:
             assert deletion_policy in self.DELETION_POLICIES
@@ -126,18 +127,29 @@ class ResourceDefinitionCore(object):
         This returns a new resource definition, with all of the functions
         parsed in the context of the specified stack and template.
         """
-        assert not getattr(self, '_frozen', False), \
-            "Cannot re-parse a frozen definition"
+        assert not getattr(self, '_frozen', False
+                           ), "Cannot re-parse a frozen definition"
 
         def reparse_snippet(snippet):
             return template.parse(stack, copy.deepcopy(snippet))
 
-        return type(self)(self.name, self.resource_type,
-                          reparse_snippet(self._properties),
-                          reparse_snippet(self._metadata),
-                          reparse_snippet(self._depends),
-                          reparse_snippet(self._deletion_policy),
-                          reparse_snippet(self._update_policy))
+        return type(self)(
+            self.name, self.resource_type,
+            properties=reparse_snippet(self._properties),
+            metadata=reparse_snippet(self._metadata),
+            depends=reparse_snippet(self._depends),
+            deletion_policy=reparse_snippet(self._deletion_policy),
+            update_policy=reparse_snippet(self._update_policy))
+
+    def dep_attrs(self, resource_name):
+        """
+        Return an iterator over dependent attributes for specified
+        resource_name in resources' properties and metadata fields.
+        """
+        return itertools.chain(function.dep_attrs(self._properties,
+                                                  resource_name),
+                               function.dep_attrs(self._metadata,
+                                                  resource_name))
 
     def dependencies(self, stack):
         """
@@ -157,8 +169,7 @@ class ResourceDefinitionCore(object):
                                                        True),
                                      function.dependencies(data, datapath))
 
-        explicit_depends = [] if self._depends is None else self._depends
-        return itertools.chain((get_resource(dep) for dep in explicit_depends),
+        return itertools.chain((get_resource(dep) for dep in self._depends),
                                strict_func_deps(self._properties,
                                                 path(PROPERTIES)),
                                strict_func_deps(self._metadata,
@@ -172,7 +183,8 @@ class ResourceDefinitionCore(object):
         require a context to validate constraints.
         """
         return properties.Properties(schema, self._properties or {},
-                                     function.resolve, self.name, context)
+                                     function.resolve, self.name, context,
+                                     section=PROPERTIES)
 
     def deletion_policy(self):
         """
@@ -190,7 +202,8 @@ class ResourceDefinitionCore(object):
         require a context to validate constraints.
         """
         return properties.Properties(schema, self._update_policy or {},
-                                     function.resolve, self.name, context)
+                                     function.resolve, self.name, context,
+                                     section=UPDATE_POLICY)
 
     def metadata(self):
         """
@@ -393,7 +406,7 @@ def _hash_data(data):
     if isinstance(data, function.Function):
         data = copy.deepcopy(data)
 
-    if not isinstance(data, basestring):
+    if not isinstance(data, six.string_types):
         if isinstance(data, collections.Sequence):
             return hash(tuple(_hash_data(d) for d in data))
 
