@@ -33,6 +33,8 @@ from heat.common.i18n import _LW
 from heat.common import identifier
 from heat.common import lifecycle_plugin_utils
 from heat.engine import dependencies
+from heat.engine import environment
+from heat.engine import event
 from heat.engine import function
 from heat.engine.notification import stack as notification
 from heat.engine import parameter_groups as param_groups
@@ -112,7 +114,7 @@ class Stack(collections.Mapping):
         self.timeout_mins = timeout_mins
         self.disable_rollback = disable_rollback
         self.parent_resource_name = parent_resource
-        self._parent_resource = None
+        self._parent_stack = None
         self._resources = None
         self._dependencies = None
         self._access_allowed_handlers = {}
@@ -161,19 +163,18 @@ class Stack(collections.Mapping):
 
         Note: this should only be used by "Fn::ResourceFacade"
         """
-        if self._parent_resource is not None:
-            return self._parent_resource
+        if self._parent_stack is None:
+            # we need both parent name and owner id.
+            if self.parent_resource_name is None or self.owner_id is None:
+                return None
 
-        # we need both parent name and owner id.
-        if self.parent_resource_name is None or self.owner_id is None:
-            return None
+            try:
+                owner = self.load(self.context, stack_id=self.owner_id)
+            except exception.NotFound:
+                return None
+            self._parent_stack = owner
 
-        try:
-            owner = self.load(self.context, stack_id=self.owner_id)
-        except exception.NotFound:
-            return None
-        self._parent_resource = owner[self.parent_resource_name]
-        return self._parent_resource
+        return self._parent_stack[self.parent_resource_name]
 
     def stored_context(self):
         if self.user_creds_id:
@@ -243,6 +244,11 @@ class Stack(collections.Mapping):
     def reset_dependencies(self):
         self._dependencies = None
 
+    def root_stack_id(self):
+        if not self.owner_id:
+            return self.id
+        return stack_object.Stack.get_root_id(self.context, self.id)
+
     @property
     def root_stack(self):
         '''
@@ -643,6 +649,14 @@ class Stack(collections.Mapping):
         '''
         return any(res.requires_deferred_auth for res in self.values())
 
+    def _add_event(self, action, status, reason):
+        '''Add a state change event to the database.'''
+        ev = event.Event(self.context, self, action, status, reason,
+                         self.id, {},
+                         self.name, 'OS::Heat::Stack')
+
+        ev.store()
+
     @profiler.trace('Stack.state_set', hide_args=False)
     def state_set(self, action, status, reason):
         '''Update the stack state in the database.'''
@@ -671,6 +685,7 @@ class Stack(collections.Mapping):
                       'name': self.name,
                       'reason': reason})
             notification.send(self)
+            self._add_event(action, status, reason)
 
     @property
     def state(self):
@@ -1252,9 +1267,14 @@ class Stack(collections.Mapping):
             return
         self.updated_time = datetime.datetime.utcnow()
 
-        template = tmpl.Template(snapshot.data['template'], env=self.env)
+        env = environment.Environment(snapshot.data['environment'])
+        template = tmpl.Template(snapshot.data['template'], env=env)
+        newstack = self.__class__(self.context, self.name, template,
+                                  timeout_mins=self.timeout_mins,
+                                  disable_rollback=self.disable_rollback)
 
-        for name, defn in six.iteritems(template.resource_definitions(self)):
+        for name, defn in six.iteritems(
+                template.resource_definitions(newstack)):
             rsrc = resource.Resource(name, defn, self)
             data = snapshot.data['resources'].get(name)
             handle_restore = getattr(rsrc, 'handle_restore', None)
@@ -1262,9 +1282,6 @@ class Stack(collections.Mapping):
                 defn = handle_restore(defn, data)
             template.add_resource(defn, name)
 
-        newstack = self.__class__(self.context, self.name, template,
-                                  timeout_mins=self.timeout_mins,
-                                  disable_rollback=self.disable_rollback)
         newstack.parameters.set_stack_id(self.identifier())
 
         updater = scheduler.TaskRunner(self.update_task, newstack,
