@@ -15,12 +15,15 @@ import abc
 import collections
 import copy
 import functools
+
+from oslo_log import log as logging
 import six
 from stevedore import extension
 
 from heat.common import exception
-from heat.db import api as db_api
-from heat.openstack.common import log as logging
+from heat.common.i18n import _
+from heat.engine import environment
+from heat.objects import raw_template as template_object
 
 LOG = logging.getLogger(__name__)
 
@@ -33,7 +36,7 @@ _template_classes = None
 def get_version(template_data, available_versions):
     version_keys = set(key for key, version in available_versions)
     candidate_keys = set(k for k, v in six.iteritems(template_data) if
-                         isinstance(v, basestring))
+                         isinstance(v, six.string_types))
 
     keys_present = version_keys & candidate_keys
 
@@ -52,7 +55,6 @@ def _get_template_extension_manager():
     return extension.ExtensionManager(
         namespace='heat.templates',
         invoke_on_load=False,
-        verify_requirements=False,
         on_load_failure_callback=raise_extension_exception)
 
 
@@ -65,13 +67,6 @@ class TemplatePluginNotRegistered(exception.HeatException):
 
 
 def get_template_class(template_data):
-    global _template_classes
-
-    if _template_classes is None:
-        mgr = _get_template_extension_manager()
-        _template_classes = dict((tuple(name.split('.')), mgr[name].plugin)
-                                 for name in mgr.names())
-
     available_versions = _template_classes.keys()
     version = get_version(template_data, available_versions)
     version_type = version[0]
@@ -97,6 +92,12 @@ class Template(collections.Mapping):
 
     def __new__(cls, template, *args, **kwargs):
         '''Create a new Template of the appropriate class.'''
+        global _template_classes
+
+        if _template_classes is None:
+            mgr = _get_template_extension_manager()
+            _template_classes = dict((tuple(name.split('.')), mgr[name].plugin)
+                                     for name in mgr.names())
 
         if cls != Template:
             TemplateClass = cls
@@ -105,7 +106,7 @@ class Template(collections.Mapping):
 
         return super(Template, cls).__new__(TemplateClass)
 
-    def __init__(self, template, template_id=None, files=None):
+    def __init__(self, template, template_id=None, files=None, env=None):
         '''
         Initialise the template with a JSON object and a set of Parameters
         '''
@@ -113,29 +114,33 @@ class Template(collections.Mapping):
         self.t = template
         self.files = files or {}
         self.maps = self[self.MAPPINGS]
+        self.env = env or environment.Environment({})
         self.version = get_version(self.t, _template_classes.keys())
 
     def __deepcopy__(self, memo):
-        return Template(copy.deepcopy(self.t, memo), files=self.files)
+        return Template(copy.deepcopy(self.t, memo), files=self.files,
+                        env=self.env)
 
     @classmethod
     def load(cls, context, template_id, t=None):
         '''Retrieve a Template with the given ID from the database.'''
         if t is None:
-            t = db_api.raw_template_get(context, template_id)
-        return cls(t.template, template_id=template_id, files=t.files)
+            t = template_object.RawTemplate.get_by_id(context, template_id)
+        env = environment.Environment(t.environment)
+        return cls(t.template, template_id=template_id, files=t.files, env=env)
 
     def store(self, context=None):
         '''Store the Template in the database and return its ID.'''
         rt = {
             'template': self.t,
-            'files': self.files
+            'files': self.files,
+            'environment': self.env.user_env_as_dict()
         }
         if self.id is None:
-            new_rt = db_api.raw_template_create(context, rt)
+            new_rt = template_object.RawTemplate.create(context, rt)
             self.id = new_rt.id
         else:
-            db_api.raw_template_update(context, self.id, rt)
+            template_object.RawTemplate.update_by_id(context, self.id, rt)
         return self.id
 
     def __iter__(self):
@@ -148,15 +153,48 @@ class Template(collections.Mapping):
         return len(self.SECTIONS) - len(self.SECTIONS_NO_DIRECT_ACCESS)
 
     @abc.abstractmethod
-    def param_schemata(self):
+    def param_schemata(self, param_defaults=None):
         '''Return a dict of parameters.Schema objects for the parameters.'''
         pass
 
     @abc.abstractmethod
-    def parameters(self, stack_identifier, user_params):
+    def get_section_name(self, section):
+        """Return a correct section name."""
+        pass
+
+    @abc.abstractmethod
+    def parameters(self, stack_identifier, user_params, param_defaults=None):
         '''Return a parameters.Parameters object for the stack.'''
         pass
 
+    @classmethod
+    def validate_resource_key_type(cls, key, valid_types, typename,
+                                   allowed_keys, rsrc_name, rsrc_data):
+        """Validation type of the specific resource key.
+
+        Used in validate_resource_definition and check correctness of
+        key's type.
+        """
+        if key not in allowed_keys:
+            raise ValueError(_('"%s" is not a valid '
+                               'keyword inside a resource '
+                               'definition') % key)
+        if key in rsrc_data:
+            if not isinstance(rsrc_data.get(key), valid_types):
+                args = {'name': rsrc_name, 'key': key,
+                        'typename': typename}
+                message = _('Resource %(name)s %(key)s type '
+                            'must be %(typename)s') % args
+                raise TypeError(message)
+            return True
+        else:
+            return False
+
+    @abc.abstractmethod
+    def validate_resource_definitions(self, stack):
+        """Check section's type of ResourceDefinitions."""
+        pass
+
     @abc.abstractmethod
     def resource_definitions(self, stack):
         '''Return a dictionary of ResourceDefinition objects.'''
@@ -196,16 +234,13 @@ class Template(collections.Mapping):
         # check resources
         for res in self[self.RESOURCES].values():
             try:
-                if not res.get('Type'):
-                    message = _('Every Resource object must '
-                                'contain a Type member.')
+                if not res or not res.get('Type'):
+                    message = _('Each Resource must contain '
+                                'a Type key.')
                     raise exception.StackValidationFailed(message=message)
             except AttributeError:
-                type_res = type(res)
-                if isinstance(res, unicode):
-                    type_res = "string"
                 message = _('Resources must contain Resource. '
-                            'Found a [%s] instead') % type_res
+                            'Found a [%s] instead') % type(res)
                 raise exception.StackValidationFailed(message=message)
 
 
@@ -219,7 +254,7 @@ def parse(functions, stack, snippet):
             if Func is not None:
                 return Func(stack, fn_name, recurse(args))
         return dict((k, recurse(v)) for k, v in six.iteritems(snippet))
-    elif (not isinstance(snippet, basestring) and
+    elif (not isinstance(snippet, six.string_types) and
           isinstance(snippet, collections.Iterable)):
         return [recurse(v) for v in snippet]
     else:
