@@ -12,29 +12,29 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""WatchRule object."""
+"""
+WatchRule object
+"""
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
-from heat.db.sqlalchemy import api as db_api
-from heat.objects import base as heat_base
+from heat.db import api as db_api
 from heat.objects import fields as heat_fields
+from heat.objects import stack
 from heat.objects import watch_data
 
 
-class WatchRule(
-        heat_base.HeatObject,
-        base.VersionedObjectDictCompat,
-):
+class WatchRule(base.VersionedObject, base.VersionedObjectDictCompat):
 
     fields = {
-        'id': fields.IntegerField(),
+        'id': fields.IntegerField(nullable=False),
         'name': fields.StringField(nullable=True),
         'rule': heat_fields.JsonField(nullable=True),
         'state': fields.StringField(nullable=True),
         'last_evaluated': fields.DateTimeField(nullable=True),
-        'stack_id': fields.StringField(),
+        'stack_id': fields.StringField(nullable=False),
+        'stack': fields.ObjectField(stack.Stack),
         'watch_data': fields.ListOfObjectsField(watch_data.WatchData),
         'created_at': fields.DateTimeField(read_only=True),
         'updated_at': fields.DateTimeField(nullable=True),
@@ -43,7 +43,10 @@ class WatchRule(
     @staticmethod
     def _from_db_object(context, rule, db_rule):
         for field in rule.fields:
-            if field == 'watch_data':
+            if field == 'stack':
+                rule[field] = stack.Stack._from_db_object(
+                    context, stack.Stack(), db_rule[field])
+            elif field == 'watch_data':
                 rule[field] = watch_data.WatchData.get_all_by_watch_rule_id(
                     context, db_rule['id'])
             else:
