@@ -10,26 +10,31 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
 import json
 
 import mock
-from oslo_utils import timeutils
+from oslo_config import cfg
 import six
 
 from heat.common import exception
 from heat.common import grouputils
 from heat.common import template_format
+from heat.engine import function
 from heat.engine import resource
 from heat.engine import rsrc_defn
 from heat.tests.autoscaling import inline_templates
 from heat.tests import common
+from heat.tests import generic_resource
 from heat.tests import utils
 
 
 class TestAutoScalingGroupValidation(common.HeatTestCase):
     def setUp(self):
         super(TestAutoScalingGroupValidation, self).setUp()
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
         self.parsed = template_format.parse(inline_templates.as_heat_template)
 
     def test_invalid_min_size(self):
@@ -44,13 +49,6 @@ class TestAutoScalingGroupValidation(common.HeatTestCase):
         self.assertRaises(exception.StackValidationFailed,
                           stack['my-group'].validate)
 
-    def test_validate_reference_attr_with_none_ref(self):
-        stack = utils.parse_stack(self.parsed)
-        group = stack['my-group']
-        self.patchobject(group, 'referenced_attrs',
-                         return_value=set([('something', None)]))
-        self.assertIsNone(group.validate())
-
 
 class TestScalingGroupTags(common.HeatTestCase):
     def setUp(self):
@@ -93,6 +91,11 @@ class TestInitialGroupSize(common.HeatTestCase):
         ('14n', dict(mins=1, maxs=4, desired=None, expected=1)),
     ]
 
+    def setUp(self):
+        super(TestInitialGroupSize, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+
     def test_initial_size(self):
         t = template_format.parse(inline_templates.as_heat_template)
         properties = t['resources']['my-group']['properties']
@@ -109,6 +112,10 @@ class TestInitialGroupSize(common.HeatTestCase):
 class TestGroupAdjust(common.HeatTestCase):
     def setUp(self):
         super(TestGroupAdjust, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
 
         t = template_format.parse(inline_templates.as_heat_template)
         self.stack = utils.parse_stack(t, params=inline_templates.as_params)
@@ -118,107 +125,52 @@ class TestGroupAdjust(common.HeatTestCase):
         self.stub_SnapshotConstraint_validate()
         self.assertIsNone(self.group.validate())
 
-    def test_group_metadata_reset(self):
-        self.group.state_set('CREATE', 'COMPLETE')
-        metadata = {'scaling_in_progress': True}
-        self.group.metadata_set(metadata)
-        self.group.handle_metadata_reset()
-
-        new_metadata = self.group.metadata_get()
-        self.assertEqual({'scaling_in_progress': False}, new_metadata)
-
     def test_scaling_policy_cooldown_toosoon(self):
-        dont_call = self.patchobject(self.group, 'resize')
-        self.patchobject(self.group, '_check_scaling_allowed',
-                         side_effect=resource.NoActionRequired)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group.adjust, 1)
+        """If _cooldown_inprogress() returns True don't progress."""
+
+        dont_call = self.patchobject(grouputils, 'get_members')
+        with mock.patch.object(self.group, '_cooldown_inprogress',
+                               return_value=True):
+            self.group.adjust(1)
         self.assertEqual([], dont_call.call_args_list)
 
     def test_scaling_same_capacity(self):
-        """Don't resize when capacity is the same."""
+        """Alway resize even if the capacity is the same."""
         self.patchobject(grouputils, 'get_size', return_value=3)
         resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
+        cd_stamp = self.patchobject(self.group, '_cooldown_timestamp')
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.assertRaises(resource.NoActionRequired,
-                          self.group.adjust, 3,
-                          adjustment_type='ExactCapacity')
-        expected_notifies = []
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        self.assertEqual(0, resize.call_count)
-        self.assertEqual(0, finished_scaling.call_count)
-
-    def test_scale_up_min_adjustment(self):
-        self.patchobject(grouputils, 'get_size', return_value=1)
-        resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.group.adjust(33, adjustment_type='PercentChangeInCapacity',
-                          min_adjustment_step=2)
-
-        expected_notifies = [
-            mock.call(
-                capacity=1, suffix='start',
-                adjustment_type='PercentChangeInCapacity',
-                groupname=u'my-group',
-                message=u'Start resizing the group my-group',
-                adjustment=33,
-                stack=self.group.stack),
-            mock.call(
-                capacity=3, suffix='end',
-                adjustment_type='PercentChangeInCapacity',
-                groupname=u'my-group',
-                message=u'End resizing the group my-group',
-                adjustment=33,
-                stack=self.group.stack)]
-
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        resize.assert_called_once_with(3)
-        finished_scaling.assert_called_once_with(
-            None,
-            'PercentChangeInCapacity : 33',
-            size_changed=True)
-
-    def test_scale_down_min_adjustment(self):
-        self.patchobject(grouputils, 'get_size', return_value=3)
-        resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.group.adjust(-33, adjustment_type='PercentChangeInCapacity',
-                          min_adjustment_step=2)
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
+        self.group.adjust(3, adjustment_type='ExactCapacity')
 
         expected_notifies = [
             mock.call(
                 capacity=3, suffix='start',
-                adjustment_type='PercentChangeInCapacity',
+                adjustment_type='ExactCapacity',
                 groupname=u'my-group',
                 message=u'Start resizing the group my-group',
-                adjustment=-33,
+                adjustment=3,
                 stack=self.group.stack),
             mock.call(
-                capacity=1, suffix='end',
-                adjustment_type='PercentChangeInCapacity',
+                capacity=3, suffix='end',
+                adjustment_type='ExactCapacity',
                 groupname=u'my-group',
                 message=u'End resizing the group my-group',
-                adjustment=-33,
+                adjustment=3,
                 stack=self.group.stack)]
 
         self.assertEqual(expected_notifies, notify.call_args_list)
-        resize.assert_called_once_with(1)
-        finished_scaling.assert_called_once_with(
-            None,
-            'PercentChangeInCapacity : -33',
-            size_changed=True)
+        resize.assert_called_once_with(3)
+        cd_stamp.assert_called_once_with('ExactCapacity : 3')
 
     def test_scaling_policy_cooldown_ok(self):
-        self.patchobject(grouputils, 'get_size', return_value=0)
+        self.patchobject(grouputils, 'get_members', return_value=[])
         resize = self.patchobject(self.group, 'resize')
-        finished_scaling = self.patchobject(self.group, '_finished_scaling')
+        cd_stamp = self.patchobject(self.group, '_cooldown_timestamp')
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
         self.group.adjust(1)
 
         expected_notifies = [
@@ -238,18 +190,15 @@ class TestGroupAdjust(common.HeatTestCase):
 
         self.assertEqual(expected_notifies, notify.call_args_list)
         resize.assert_called_once_with(1)
-        finished_scaling.assert_called_once_with(None,
-                                                 'ChangeInCapacity : 1',
-                                                 size_changed=True)
-        grouputils.get_size.assert_called_once_with(self.group)
+        cd_stamp.assert_called_once_with('ChangeInCapacity : 1')
 
     def test_scaling_policy_resize_fail(self):
-        self.patchobject(grouputils, 'get_size', return_value=0)
+        self.patchobject(grouputils, 'get_members', return_value=[])
         self.patchobject(self.group, 'resize',
                          side_effect=ValueError('test error'))
         notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.patchobject(self.group, '_finished_scaling')
+        self.patchobject(self.group, '_cooldown_inprogress',
+                         return_value=False)
         self.assertRaises(ValueError, self.group.adjust, 1)
 
         expected_notifies = [
@@ -269,45 +218,15 @@ class TestGroupAdjust(common.HeatTestCase):
                 stack=self.group.stack)]
 
         self.assertEqual(expected_notifies, notify.call_args_list)
-        grouputils.get_size.assert_called_with(self.group)
-
-    def test_notification_send_if_resize_failed(self):
-        """If resize failed, the capacity of group might have been changed"""
-        self.patchobject(grouputils, 'get_size', side_effect=[3, 4])
-        self.patchobject(self.group, 'resize',
-                         side_effect=ValueError('test error'))
-        notify = self.patch('heat.engine.notification.autoscaling.send')
-        self.patchobject(self.group, '_check_scaling_allowed')
-        self.patchobject(self.group, '_finished_scaling')
-
-        self.assertRaises(ValueError, self.group.adjust,
-                          5, adjustment_type='ExactCapacity')
-
-        expected_notifies = [
-            mock.call(
-                capacity=3, suffix='start',
-                adjustment_type='ExactCapacity',
-                groupname='my-group',
-                message='Start resizing the group my-group',
-                adjustment=5,
-                stack=self.group.stack),
-            mock.call(
-                capacity=4, suffix='error',
-                adjustment_type='ExactCapacity',
-                groupname='my-group',
-                message=u'test error',
-                adjustment=5,
-                stack=self.group.stack)]
-
-        self.assertEqual(expected_notifies, notify.call_args_list)
-        self.group.resize.assert_called_once_with(5)
-        grouputils.get_size.assert_has_calls([mock.call(self.group),
-                                              mock.call(self.group)])
 
 
 class TestGroupCrud(common.HeatTestCase):
     def setUp(self):
         super(TestGroupCrud, self).setUp()
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
         self.stub_ImageConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_SnapshotConstraint_validate()
@@ -328,11 +247,9 @@ class TestGroupCrud(common.HeatTestCase):
 
     def test_handle_update_desired_cap(self):
         self.group._try_rolling_update = mock.Mock(return_value=None)
-        self.group.resize = mock.Mock(return_value=None)
+        self.group.adjust = mock.Mock(return_value=None)
 
-        props = {'desired_capacity': 4,
-                 'min_size': 0,
-                 'max_size': 6}
+        props = {'desired_capacity': 4}
         defn = rsrc_defn.ResourceDefinition(
             'nopayload',
             'OS::Heat::AutoScalingGroup',
@@ -340,17 +257,17 @@ class TestGroupCrud(common.HeatTestCase):
 
         self.group.handle_update(defn, None, props)
 
-        self.group.resize.assert_called_once_with(4)
+        self.group.adjust.assert_called_once_with(
+            4, adjustment_type='ExactCapacity')
         self.group._try_rolling_update.assert_called_once_with(props)
 
     def test_handle_update_desired_nocap(self):
         self.group._try_rolling_update = mock.Mock(return_value=None)
-        self.group.resize = mock.Mock(return_value=None)
+        self.group.adjust = mock.Mock(return_value=None)
         get_size = self.patchobject(grouputils, 'get_size')
         get_size.return_value = 6
 
-        props = {'min_size': 0,
-                 'max_size': 6}
+        props = {'Tags': []}
         defn = rsrc_defn.ResourceDefinition(
             'nopayload',
             'OS::Heat::AutoScalingGroup',
@@ -358,13 +275,14 @@ class TestGroupCrud(common.HeatTestCase):
 
         self.group.handle_update(defn, None, props)
 
-        self.group.resize.assert_called_once_with(6)
+        self.group.adjust.assert_called_once_with(
+            6, adjustment_type='ExactCapacity')
         self.group._try_rolling_update.assert_called_once_with(props)
 
     def test_update_in_failed(self):
         self.group.state_set('CREATE', 'FAILED')
         # to update the failed asg
-        self.group.resize = mock.Mock(return_value=None)
+        self.group.adjust = mock.Mock(return_value=None)
 
         new_defn = rsrc_defn.ResourceDefinition(
             'asg', 'OS::Heat::AutoScalingGroup',
@@ -379,12 +297,17 @@ class TestGroupCrud(common.HeatTestCase):
                   'Foo': 'hello'}}})
 
         self.group.handle_update(new_defn, None, None)
-        self.group.resize.assert_called_once_with(2)
+        self.group.adjust.assert_called_once_with(
+            2, adjustment_type='ExactCapacity')
 
 
 class HeatScalingGroupAttrTest(common.HeatTestCase):
     def setUp(self):
         super(HeatScalingGroupAttrTest, self).setUp()
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://server.test:8000/v1/waitcondition')
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
 
         t = template_format.parse(inline_templates.as_heat_template)
         self.stack = utils.parse_stack(t, params=inline_templates.as_params)
@@ -392,9 +315,7 @@ class HeatScalingGroupAttrTest(common.HeatTestCase):
         self.assertIsNone(self.group.validate())
 
     def test_no_instance_list(self):
-        """Tests inheritance of InstanceList attribute.
-
-        The InstanceList attribute is not inherited from
+        """The InstanceList attribute is not inherited from
         AutoScalingResourceGroup's superclasses.
         """
         self.assertRaises(exception.InvalidTemplateAttribute,
@@ -413,37 +334,6 @@ class HeatScalingGroupAttrTest(common.HeatTestCase):
 
         self.assertEqual(output, self.group.FnGetAtt('outputs_list', 'Bar'))
 
-    def test_output_refs(self):
-        # Setup
-        mock_get = self.patchobject(grouputils, 'get_member_refids')
-        mock_get.return_value = ['resource-1', 'resource-2']
-
-        # Test
-        found = self.group.FnGetAtt('refs')
-
-        # Verify
-        expected = ['resource-1', 'resource-2']
-        self.assertEqual(expected, found)
-        mock_get.assert_called_once_with(self.group)
-
-    def test_output_refs_map(self):
-        # Setup
-        mock_members = self.patchobject(grouputils, 'get_members')
-        members = [mock.MagicMock(), mock.MagicMock()]
-        members[0].name = 'resource-1-name'
-        members[0].resource_id = 'resource-1-id'
-        members[1].name = 'resource-2-name'
-        members[1].resource_id = 'resource-2-id'
-        mock_members.return_value = members
-
-        # Test
-        found = self.group.FnGetAtt('refs_map')
-
-        # Verify
-        expected = {'resource-1-name': 'resource-1-id',
-                    'resource-2-name': 'resource-2-id'}
-        self.assertEqual(expected, found)
-
     def test_output_attribute_dict(self):
         mock_members = self.patchobject(grouputils, 'get_members')
         members = []
@@ -471,7 +361,6 @@ class HeatScalingGroupAttrTest(common.HeatTestCase):
 
     def test_index_dotted_attribute(self):
         mock_members = self.patchobject(grouputils, 'get_members')
-        self.group.nested = mock.Mock()
         members = []
         output = []
         for ip_ex in six.moves.range(0, 2):
@@ -483,7 +372,7 @@ class HeatScalingGroupAttrTest(common.HeatTestCase):
         mock_members.return_value = members
         self.assertEqual(output[0], self.group.FnGetAtt('resource.0', 'Bar'))
         self.assertEqual(output[1], self.group.FnGetAtt('resource.1.Bar'))
-        self.assertRaises(exception.NotFound,
+        self.assertRaises(exception.InvalidTemplateAttribute,
                           self.group.FnGetAtt, 'resource.2')
 
 
@@ -516,6 +405,10 @@ class RollingUpdatePolicyTest(common.HeatTestCase):
     def setUp(self):
         super(RollingUpdatePolicyTest, self).setUp()
         self.stub_keystoneclient(username='test_stack.CfnLBUser')
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://127.0.0.1:8000/v1/waitcondition')
 
     def test_parse_without_update_policy(self):
         tmpl = template_format.parse(inline_templates.as_heat_template)
@@ -538,7 +431,7 @@ class RollingUpdatePolicyTest(common.HeatTestCase):
         tmpl_batch_sz = int(tmpl_policy['max_batch_size'])
         policy = stack['my-group'].properties['rolling_updates']
         self.assertTrue(policy)
-        self.assertEqual(3, len(policy))
+        self.assertTrue(len(policy) == 3)
         self.assertEqual(1, int(policy['min_in_service']))
         self.assertEqual(tmpl_batch_sz, int(policy['max_batch_size']))
         self.assertEqual(1, policy['pause_time'])
@@ -576,6 +469,10 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
     def setUp(self):
         super(RollingUpdatePolicyDiffTest, self).setUp()
         self.stub_keystoneclient(username='test_stack.CfnLBUser')
+        resource._register_class('ResourceWithPropsAndAttrs',
+                                 generic_resource.ResourceWithPropsAndAttrs)
+        cfg.CONF.set_default('heat_waitcondition_server_url',
+                             'http://127.0.0.1:8000/v1/waitcondition')
 
     def validate_update_policy_diff(self, current, updated):
         # load current stack
@@ -584,7 +481,7 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
 
         # get the json snippet for the current InstanceGroup resource
         current_grp = current_stack['my-group']
-        current_snippets = dict((n, r.frozen_definition())
+        current_snippets = dict((n, r.parsed_template())
                                 for n, r in current_stack.items())
         current_grp_json = current_snippets[current_grp.name]
 
@@ -595,7 +492,7 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
         # get the updated json snippet for the InstanceGroup resource in the
         # context of the current stack
         updated_grp = updated_stack['my-group']
-        updated_grp_json = updated_grp.t.freeze()
+        updated_grp_json = function.resolve(updated_grp.t)
 
         # identify the template difference
         tmpl_diff = updated_grp.update_template_diff(
@@ -603,11 +500,17 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
         updated_policy = (updated_grp.properties['rolling_updates']
                           if 'rolling_updates' in updated_grp.properties.data
                           else None)
-        self.assertTrue(tmpl_diff.properties_changed())
+        self.assertEqual(updated_policy,
+                         tmpl_diff['Properties'].get('rolling_updates'))
 
+        # test application of the new update policy in handle_update
+        update_snippet = rsrc_defn.ResourceDefinition(
+            current_grp.name,
+            current_grp.type(),
+            properties=updated_grp.t['Properties'])
         current_grp._try_rolling_update = mock.MagicMock()
-        current_grp.resize = mock.MagicMock()
-        current_grp.handle_update(updated_grp_json, tmpl_diff, None)
+        current_grp.adjust = mock.MagicMock()
+        current_grp.handle_update(update_snippet, tmpl_diff, None)
         if updated_policy is None:
             self.assertIsNone(
                 current_grp.properties.data.get('rolling_updates'))
@@ -631,157 +534,3 @@ class RollingUpdatePolicyDiffTest(common.HeatTestCase):
     def test_update_policy_removed(self):
         self.validate_update_policy_diff(asg_tmpl_with_updt_policy(),
                                          inline_templates.as_heat_template)
-
-
-class IncorrectUpdatePolicyTest(common.HeatTestCase):
-    def setUp(self):
-        super(IncorrectUpdatePolicyTest, self).setUp()
-        self.stub_keystoneclient(username='test_stack.CfnLBUser')
-
-    def test_with_update_policy_aws(self):
-        t = template_format.parse(inline_templates.as_heat_template)
-        ag = t['resources']['my-group']
-        ag["update_policy"] = {"AutoScalingRollingUpdate": {
-            "MinInstancesInService": "1",
-            "MaxBatchSize": "2",
-            "PauseTime": "PT1S"
-        }}
-        tmpl = template_format.parse(json.dumps(t))
-        stack = utils.parse_stack(tmpl)
-        exc = self.assertRaises(exception.StackValidationFailed,
-                                stack.validate)
-        self.assertIn('Unknown Property AutoScalingRollingUpdate',
-                      six.text_type(exc))
-
-    def test_with_update_policy_inst_group(self):
-        t = template_format.parse(inline_templates.as_heat_template)
-        ag = t['resources']['my-group']
-        ag["update_policy"] = {"RollingUpdate": {
-            "MinInstancesInService": "1",
-            "MaxBatchSize": "2",
-            "PauseTime": "PT1S"
-        }}
-        tmpl = template_format.parse(json.dumps(t))
-        stack = utils.parse_stack(tmpl)
-        exc = self.assertRaises(exception.StackValidationFailed,
-                                stack.validate)
-        self.assertIn('Unknown Property RollingUpdate', six.text_type(exc))
-
-
-class TestCooldownMixin(common.HeatTestCase):
-    def setUp(self):
-        super(TestCooldownMixin, self).setUp()
-        t = template_format.parse(inline_templates.as_heat_template)
-        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.stack.store()
-        self.group = self.stack['my-group']
-        self.group.state_set('CREATE', 'COMPLETE')
-
-    def test_cooldown_is_in_progress_toosoon(self):
-        cooldown_end = timeutils.utcnow() + datetime.timedelta(seconds=60)
-        previous_meta = {'cooldown_end': {
-            cooldown_end.isoformat(): 'change_in_capacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_cooldown_is_in_progress_toosoon_legacy(self):
-        now = timeutils.utcnow()
-        previous_meta = {'cooldown': {
-            now.isoformat(): 'change_in_capacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_cooldown_is_in_progress_scaling_unfinished(self):
-        previous_meta = {'scaling_in_progress': True}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertRaises(resource.NoActionRequired,
-                          self.group._check_scaling_allowed,
-                          60)
-
-    def test_cooldown_not_in_progress_legacy(self):
-        awhile_ago = timeutils.utcnow() - datetime.timedelta(seconds=100)
-        previous_meta = {
-            'cooldown': {
-                awhile_ago.isoformat(): 'change_in_capacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-
-    def test_cooldown_not_in_progress(self):
-        awhile_after = timeutils.utcnow() + datetime.timedelta(seconds=60)
-        previous_meta = {
-            'cooldown_end': {
-                awhile_after.isoformat(): 'change_in_capacity : 1'
-            },
-            'scaling_in_progress': False
-        }
-        timeutils.set_time_override()
-        timeutils.advance_time_seconds(100)
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-        timeutils.clear_time_override()
-
-    def test_scaling_policy_cooldown_zero(self):
-        now = timeutils.utcnow()
-        previous_meta = {'cooldown_end': {
-            now.isoformat(): 'change_in_capacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(0))
-
-    def test_scaling_policy_cooldown_none(self):
-        now = timeutils.utcnow()
-        previous_meta = {'cooldown_end': {
-            now.isoformat(): 'change_in_capacity : 1'}}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(None))
-
-    def test_no_cooldown_no_scaling_in_progress(self):
-        # no cooldown entry in the metadata
-        awhile_ago = timeutils.utcnow() - datetime.timedelta(seconds=100)
-        previous_meta = {'scaling_in_progress': False,
-                         awhile_ago.isoformat(): 'change_in_capacity : 1'}
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        self.assertIsNone(self.group._check_scaling_allowed(60))
-
-    def test_metadata_is_written(self):
-        nowish = timeutils.utcnow()
-        reason = 'cool as'
-        meta_set = self.patchobject(self.group, 'metadata_set')
-        self.patchobject(timeutils, 'utcnow', return_value=nowish)
-        self.group._finished_scaling(60, reason)
-        cooldown_end = nowish + datetime.timedelta(seconds=60)
-        meta_set.assert_called_once_with(
-            {'cooldown_end': {cooldown_end.isoformat(): reason},
-             'scaling_in_progress': False})
-
-    def test_metadata_is_written_update(self):
-        nowish = timeutils.utcnow()
-        reason = 'cool as'
-        prev_cooldown_end = nowish + datetime.timedelta(seconds=100)
-        previous_meta = {
-            'cooldown_end': {
-                prev_cooldown_end.isoformat(): 'change_in_capacity : 1'
-            }
-        }
-        self.patchobject(self.group, 'metadata_get',
-                         return_value=previous_meta)
-        meta_set = self.patchobject(self.group, 'metadata_set')
-        self.patchobject(timeutils, 'utcnow', return_value=nowish)
-        self.group._finished_scaling(60, reason)
-        meta_set.assert_called_once_with(
-            {'cooldown_end': {prev_cooldown_end.isoformat(): reason},
-             'scaling_in_progress': False})
