@@ -220,8 +220,10 @@ class TestAutoScalingGroupValidation(common.HeatTestCase):
         rsrc = stack['WebServerGroup']
         self._stub_nova_server_get(not_found=True)
         self.m.ReplayAll()
-        msg = ("Property error : WebServerGroup: InstanceId Error validating "
-               "value '5678': The server (5678) could not be found")
+        msg = ("Property error: "
+               "Resources.WebServerGroup.Properties.InstanceId: "
+               "Error validating value '5678': The server (5678) could "
+               "not be found.")
         exc = self.assertRaises(exception.StackValidationFailed,
                                 rsrc.validate)
         self.assertIn(msg, six.text_type(exc))
@@ -233,8 +235,8 @@ class TestScalingGroupTags(common.HeatTestCase):
     def setUp(self):
         super(TestScalingGroupTags, self).setUp()
         t = template_format.parse(as_template)
-        stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = stack['WebServerGroup']
+        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = self.stack['WebServerGroup']
 
     def test_tags_default(self):
         expected = [{'Key': 'metering.groupname',
@@ -297,8 +299,8 @@ class TestGroupAdjust(common.HeatTestCase):
                              'http://server.test:8000/v1/waitcondition')
 
         t = template_format.parse(as_template)
-        stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = stack['WebServerGroup']
+        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = self.stack['WebServerGroup']
         self.stub_ImageConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_SnapshotConstraint_validate()
@@ -406,8 +408,8 @@ class TestGroupCrud(common.HeatTestCase):
                              'http://server.test:8000/v1/waitcondition')
 
         t = template_format.parse(as_template)
-        stack = utils.parse_stack(t, params=inline_templates.as_params)
-        self.group = stack['WebServerGroup']
+        self.stack = utils.parse_stack(t, params=inline_templates.as_params)
+        self.group = self.stack['WebServerGroup']
         self.assertIsNone(self.group.validate())
 
     def test_handle_create(self):
