@@ -19,42 +19,52 @@ import time
 
 import fixtures
 import mox
-from oslo.config import cfg
+from oslo_config import cfg
 from oslotest import mockpatch
 import testscenarios
 import testtools
 
+from heat.common import context
 from heat.common import messaging
+from heat.engine.clients.os import cinder
 from heat.engine.clients.os import glance
 from heat.engine.clients.os import keystone
+from heat.engine.clients.os import neutron
+from heat.engine.clients.os import nova
+from heat.engine.clients.os import trove
 from heat.engine import environment
 from heat.engine import resources
-from heat.engine.resources import nova_keypair
 from heat.engine import scheduler
-from heat.engine import stack
 from heat.tests import fakes
 from heat.tests import utils
 
 
-TEST_DEFAULT_LOGLEVELS = {'migrate': logging.WARN}
+TEST_DEFAULT_LOGLEVELS = {'migrate': logging.WARN,
+                          'sqlalchemy': logging.WARN}
+_LOG_FORMAT = "%(levelname)8s [%(name)s] %(message)s"
+_TRUE_VALUES = ('True', 'true', '1', 'yes')
 
 
 class FakeLogMixin(object):
     def setup_logging(self):
         # Assign default logs to self.LOG so we can still
         # assert on heat logs.
+        default_level = logging.INFO
+        if os.environ.get('OS_DEBUG') in _TRUE_VALUES:
+            default_level = logging.DEBUG
+
         self.LOG = self.useFixture(
-            fixtures.FakeLogger(level=logging.DEBUG))
+            fixtures.FakeLogger(level=default_level, format=_LOG_FORMAT))
         base_list = set([nlog.split('.')[0]
                          for nlog in logging.Logger.manager.loggerDict])
         for base in base_list:
             if base in TEST_DEFAULT_LOGLEVELS:
                 self.useFixture(fixtures.FakeLogger(
                     level=TEST_DEFAULT_LOGLEVELS[base],
-                    name=base))
+                    name=base, format=_LOG_FORMAT))
             elif base != 'heat':
                 self.useFixture(fixtures.FakeLogger(
-                    name=base))
+                    name=base, format=_LOG_FORMAT))
 
 
 class HeatTestCase(testscenarios.WithScenarios,
@@ -62,7 +72,7 @@ class HeatTestCase(testscenarios.WithScenarios,
 
     TIME_STEP = 0.1
 
-    def setUp(self):
+    def setUp(self, mock_keystone=True):
         super(HeatTestCase, self).setUp()
         self.m = mox.Mox()
         self.addCleanup(self.m.UnsetStubs)
@@ -83,6 +93,7 @@ class HeatTestCase(testscenarios.WithScenarios,
                                'environment.d')
 
         cfg.CONF.set_default('environment_dir', env_dir)
+        cfg.CONF.set_override('error_wait_time', None)
         self.addCleanup(cfg.CONF.reset)
 
         messaging.setup("fake://", optional=True)
@@ -101,18 +112,11 @@ class HeatTestCase(testscenarios.WithScenarios,
         # use CWLiteAlarm for testing.
         resources.global_env().registry.load(
             {"AWS::CloudWatch::Alarm": "OS::Heat::CWLiteAlarm"})
-
+        if mock_keystone:
+            self.stub_keystoneclient()
         utils.setup_dummy_db()
         self.addCleanup(utils.reset_dummy_db)
 
-        cached_wait_time = stack.ERROR_WAIT_TIME
-        stack.ERROR_WAIT_TIME = None
-
-        def replace_wait_time():
-            stack.ERROR_WAIT_TIME = cached_wait_time
-
-        self.addCleanup(replace_wait_time)
-
     def stub_wallclock(self):
         """
         Overrides scheduler wallclock to speed up tests expecting timeouts.
@@ -137,6 +141,13 @@ class HeatTestCase(testscenarios.WithScenarios,
         mockfixture = self.useFixture(mockpatch.Patch(target, **kwargs))
         return mockfixture.mock
 
+    def stub_auth(self, ctx=None, **kwargs):
+        auth = self.patchobject(ctx or context.RequestContext,
+                                "_create_auth_plugin")
+        fake_auth = fakes.FakeAuth(**kwargs)
+        auth.return_value = fake_auth
+        return auth
+
     def stub_keystoneclient(self, fake_client=None, **kwargs):
         client = self.patchobject(keystone.KeystoneClientPlugin, "_create")
         fkc = fake_client or fakes.FakeKeystoneClient(**kwargs)
@@ -144,17 +155,53 @@ class HeatTestCase(testscenarios.WithScenarios,
         return fkc
 
     def stub_KeypairConstraint_validate(self):
-        self.m.StubOutWithMock(nova_keypair.KeypairConstraint, 'validate')
-        nova_keypair.KeypairConstraint.validate(
-            mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().AndReturn(True)
+        validate = self.patchobject(nova.KeypairConstraint, 'validate')
+        validate.return_value = True
 
     def stub_ImageConstraint_validate(self, num=None):
-        self.m.StubOutWithMock(glance.ImageConstraint, 'validate')
+        validate = self.patchobject(glance.ImageConstraint, 'validate')
         if num is None:
-            glance.ImageConstraint.validate(
-                mox.IgnoreArg(), mox.IgnoreArg()).MultipleTimes().\
-                AndReturn(True)
+            validate.return_value = True
         else:
-            for x in range(num):
-                glance.ImageConstraint.validate(
-                    mox.IgnoreArg(), mox.IgnoreArg()).AndReturn(True)
+            validate.side_effect = [True for x in range(num)]
+
+    def stub_FlavorConstraint_validate(self):
+        validate = self.patchobject(nova.FlavorConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_VolumeConstraint_validate(self):
+        validate = self.patchobject(cinder.VolumeConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_SnapshotConstraint_validate(self):
+        validate = self.patchobject(
+            cinder.VolumeSnapshotConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_VolumeTypeConstraint_validate(self):
+        validate = self.patchobject(cinder.VolumeTypeConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_ServerConstraint_validate(self):
+        validate = self.patchobject(nova.ServerConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_NetworkConstraint_validate(self):
+        validate = self.patchobject(neutron.NetworkConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_PortConstraint_validate(self):
+        validate = self.patchobject(neutron.PortConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_TroveFlavorConstraint_validate(self):
+        validate = self.patchobject(trove.FlavorConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_SubnetConstraint_validate(self):
+        validate = self.patchobject(neutron.SubnetConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_RouterConstraint_validate(self):
+        validate = self.patchobject(neutron.RouterConstraint, 'validate')
+        validate.return_value = True
