@@ -12,27 +12,28 @@
 #    under the License.
 
 import copy
-import six
 import uuid
 
 from glanceclient import exc as glance_exceptions
 import mock
 import mox
 from neutronclient.v2_0 import client as neutronclient
+import six
 
 from heat.common import exception
 from heat.common import template_format
+from heat.engine.clients.os import cinder
 from heat.engine.clients.os import glance
 from heat.engine.clients.os import neutron
 from heat.engine.clients.os import nova
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine import resource
-from heat.engine.resources import instance as instances
+from heat.engine.resources.aws.ec2 import instance as instances
 from heat.engine import scheduler
-from heat.tests.common import HeatTestCase
+from heat.tests import common
+from heat.tests.nova import fakes as fakes_nova
 from heat.tests import utils
-from heat.tests.v1_1 import fakes
 
 
 wp_template = '''
@@ -71,16 +72,16 @@ wp_template = '''
 '''
 
 
-class InstancesTest(HeatTestCase):
+class InstancesTest(common.HeatTestCase):
     def setUp(self):
         super(InstancesTest, self).setUp()
-        self.fc = fakes.FakeClient()
+        self.fc = fakes_nova.FakeClient()
 
     def _setup_test_stack(self, stack_name):
         t = template_format.parse(wp_template)
-        template = parser.Template(t)
+        template = parser.Template(
+            t, env=environment.Environment({'KeyName': 'test'}))
         stack = parser.Stack(utils.dummy_context(), stack_name, template,
-                             environment.Environment({'KeyName': 'test'}),
                              stack_id=str(uuid.uuid4()))
         return (template, stack)
 
@@ -96,25 +97,27 @@ class InstancesTest(HeatTestCase):
     def _get_test_template(self, stack_name, image_id=None):
         (tmpl, stack) = self._setup_test_stack(stack_name)
 
-        tmpl.t['Resources']['WebServer']['Properties']['ImageId'] = \
-            image_id or 'CentOS 5.2'
-        tmpl.t['Resources']['WebServer']['Properties']['InstanceType'] = \
-            '256 MB Server'
+        tmpl.t['Resources']['WebServer']['Properties'][
+            'ImageId'] = image_id or 'CentOS 5.2'
+        tmpl.t['Resources']['WebServer']['Properties'][
+            'InstanceType'] = '256 MB Server'
 
         return tmpl, stack
 
     def _setup_test_instance(self, return_server, name, image_id=None,
                              stub_create=True):
         stack_name = '%s_s' % name
-        tmpl, stack = self._get_test_template(stack_name, image_id)
-        resource_defns = tmpl.resource_definitions(stack)
-        instance = instances.Instance(name, resource_defns['WebServer'], stack)
+        tmpl, self.stack = self._get_test_template(stack_name, image_id)
+        resource_defns = tmpl.resource_definitions(self.stack)
+        instance = instances.Instance(name, resource_defns['WebServer'],
+                                      self.stack)
         bdm = {"vdb": "9ef5496e-7426-446a-bbc8-01f84d9c9972:snap::True"}
 
         self._mock_get_image_id_success(image_id or 'CentOS 5.2', 1)
 
         self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
         nova.NovaClientPlugin._create().AndReturn(self.fc)
+        self.stub_SnapshotConstraint_validate()
 
         if stub_create:
             self.m.StubOutWithMock(self.fc.servers, 'create')
@@ -203,6 +206,34 @@ class InstancesTest(HeatTestCase):
                                           'DeleteOnTermination': True}},
         ]))
 
+    def test_validate_Volumes_property(self):
+        stack_name = 'validate_volumes'
+        tmpl, stack = self._setup_test_stack(stack_name)
+        volumes = [{'Device': 'vdb', 'VolumeId': '1234'}]
+        wsp = tmpl.t['Resources']['WebServer']['Properties']
+        wsp['Volumes'] = volumes
+        resource_defns = tmpl.resource_definitions(stack)
+        instance = instances.Instance('validate_volumes',
+                                      resource_defns['WebServer'], stack)
+
+        self._mock_get_image_id_success('F17-x86_64-gold', 1)
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().MultipleTimes().AndReturn(self.fc)
+        self.stub_SnapshotConstraint_validate()
+        self.m.StubOutWithMock(cinder.CinderClientPlugin, 'get_volume')
+        ex = exception.VolumeNotFound(volume='1234')
+        cinder.CinderClientPlugin.get_volume('1234').AndRaise(ex)
+        self.m.ReplayAll()
+
+        exc = self.assertRaises(exception.StackValidationFailed,
+                                instance.validate)
+        self.assertIn("WebServer.Properties.Volumes[0].VolumeId: "
+                      "Error validating value '1234': The Volume "
+                      "(1234) could not be found.",
+                      six.text_type(exc))
+
+        self.m.VerifyAll()
+
     def test_validate_BlockDeviceMappings_VolumeSize_valid_str(self):
         stack_name = 'val_VolumeSize_valid'
         tmpl, stack = self._setup_test_stack(stack_name)
@@ -216,6 +247,7 @@ class InstancesTest(HeatTestCase):
                                       resource_defns['WebServer'], stack)
 
         self._mock_get_image_id_success('F17-x86_64-gold', 1)
+        self.stub_SnapshotConstraint_validate()
         self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
         nova.NovaClientPlugin._create().MultipleTimes().AndReturn(self.fc)
 
@@ -291,9 +323,10 @@ class InstancesTest(HeatTestCase):
 
         exc = self.assertRaises(exception.StackValidationFailed,
                                 instance.validate)
-        excepted_error = ('Property error : WebServer: BlockDeviceMappings '
-                          'Property error : BlockDeviceMappings: 0 Property '
-                          'error : 0: Property DeviceName not assigned')
+        excepted_error = (
+            'Property error: '
+            'Resources.WebServer.Properties.BlockDeviceMappings[0]: '
+            'Property DeviceName not assigned')
         self.assertIn(excepted_error, six.text_type(exc))
 
         self.m.VerifyAll()
@@ -351,9 +384,10 @@ class InstancesTest(HeatTestCase):
         create = scheduler.TaskRunner(instance.create)
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
-            'StackValidationFailed: Property error : WebServer: '
-            'ImageId Error validating value \'Slackware\': '
-            'The Image (Slackware) could not be found.',
+            "StackValidationFailed: resources.instance_create_image_err: "
+            "Property error: "
+            "WebServer.Properties.ImageId: Error validating value "
+            "'Slackware': The Image (Slackware) could not be found.",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -378,9 +412,10 @@ class InstancesTest(HeatTestCase):
         create = scheduler.TaskRunner(instance.create)
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
-            'StackValidationFailed: Property error : WebServer: '
-            'ImageId Multiple physical resources were '
-            'found with name (CentOS 5.2).',
+            'StackValidationFailed: resources.instance_create_image_err: '
+            'Property error: '
+            'WebServer.Properties.ImageId: Multiple physical '
+            'resources were found with name (CentOS 5.2).',
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -402,8 +437,8 @@ class InstancesTest(HeatTestCase):
         create = scheduler.TaskRunner(instance.create)
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
-            'StackValidationFailed: Property error : WebServer: '
-            'ImageId 404 (HTTP 404)',
+            'StackValidationFailed: resources.instance_create_image_err: '
+            'Property error: WebServer.Properties.ImageId: 404 (HTTP 404)',
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -445,9 +480,8 @@ class InstancesTest(HeatTestCase):
         e = self.assertRaises(resource.ResourceUnknownStatus,
                               instance.check_create_complete,
                               (return_server, self.FakeVolumeAttach()))
-        self.assertEqual(
-            'Instance is not active - Unknown status BOGUS',
-            six.text_type(e))
+        self.assertEqual('Instance is not active - Unknown status BOGUS '
+                         'due to "Unknown"', six.text_type(e))
 
     def test_instance_create_error_status(self):
         return_server = self.fc.servers.list()[1]
@@ -491,6 +525,46 @@ class InstancesTest(HeatTestCase):
 
         self.m.VerifyAll()
 
+    def test_instance_create_with_stack_scheduler_hints(self):
+        return_server = self.fc.servers.list()[1]
+        instances.cfg.CONF.set_override('stack_scheduler_hints', True)
+        # Unroll _create_test_instance, to enable check
+        # for addition of heat ids (stack id, resource name)
+        stack_name = 'test_instance_create_with_stack_scheduler_hints'
+        (t, stack) = self._get_test_template(stack_name)
+        resource_defns = t.resource_definitions(stack)
+        instance = instances.Instance('in_create_with_sched_hints',
+                                      resource_defns['WebServer'], stack)
+        bdm = {"vdb": "9ef5496e-7426-446a-bbc8-01f84d9c9972:snap::True"}
+        self._mock_get_image_id_success('CentOS 5.2', 1)
+
+        self.m.StubOutWithMock(nova.NovaClientPlugin, '_create')
+        nova.NovaClientPlugin._create().AndReturn(self.fc)
+        self.stub_SnapshotConstraint_validate()
+
+        self.m.StubOutWithMock(self.fc.servers, 'create')
+        self.fc.servers.create(
+            image=1, flavor=1, key_name='test',
+            name=utils.PhysName(
+                stack_name,
+                instance.name,
+                limit=instance.physical_resource_name_limit),
+            security_groups=None,
+            userdata=mox.IgnoreArg(),
+            scheduler_hints={'heat_root_stack_id': stack.root_stack_id(),
+                             'heat_stack_id': stack.id,
+                             'heat_stack_name': stack.name,
+                             'heat_path_in_stack': [(None, stack.name)],
+                             'heat_resource_name': instance.name,
+                             'foo': ['spam', 'ham', 'baz'], 'bar': 'eggs'},
+            meta=None, nics=None, availability_zone=None,
+            block_device_mapping=bdm).AndReturn(
+                return_server)
+        self.m.ReplayAll()
+        scheduler.TaskRunner(instance.create)()
+        self.assertTrue(instance.id > 0)
+        self.m.VerifyAll()
+
     def test_instance_validate(self):
         stack_name = 'test_instance_validate_stack'
         (tmpl, stack) = self._setup_test_stack(stack_name)
@@ -504,6 +578,7 @@ class InstancesTest(HeatTestCase):
         nova.NovaClientPlugin._create().AndReturn(self.fc)
 
         self._mock_get_image_id_success('1', 1)
+        self.stub_SnapshotConstraint_validate()
         self.m.ReplayAll()
 
         self.assertIsNone(instance.validate())
@@ -533,7 +608,7 @@ class InstancesTest(HeatTestCase):
             d2['server']['status'] = vm_delete_status
             get().AndReturn((200, d2))
         else:
-            get().AndRaise(fakes.fake_exception())
+            get().AndRaise(fakes_nova.fake_exception())
 
         self.m.ReplayAll()
 
@@ -624,7 +699,8 @@ class InstancesTest(HeatTestCase):
         updater = scheduler.TaskRunner(instance.update, update_template)
         error = self.assertRaises(exception.ResourceFailure, updater)
         self.assertEqual(
-            "Error: Resizing to 'm1.small' failed, status 'ACTIVE'",
+            "Error: resources.ud_type_f: "
+            "Resizing to 'm1.small' failed, status 'ACTIVE'",
             six.text_type(error))
         self.assertEqual((instance.UPDATE, instance.FAILED), instance.state)
         self.m.VerifyAll()
