@@ -13,13 +13,14 @@
 
 import json
 import os
-import six
 import uuid
-import yaml
 
 import mock
+import six
 
 from heat.common import exception
+from heat.common.i18n import _
+from heat.common import identifier
 from heat.common import template_format
 from heat.common import urlfetch
 from heat.engine import attributes
@@ -31,7 +32,7 @@ from heat.engine import resources
 from heat.engine.resources import template_resource
 from heat.engine import rsrc_defn
 from heat.engine import support
-from heat.tests.common import HeatTestCase
+from heat.tests import common
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
@@ -43,7 +44,7 @@ class MyCloudResource(generic_rsrc.GenericResource):
     pass
 
 
-class ProviderTemplateTest(HeatTestCase):
+class ProviderTemplateTest(common.HeatTestCase):
     def setUp(self):
         super(ProviderTemplateTest, self).setUp()
         resource._register_class('OS::ResourceType',
@@ -97,6 +98,7 @@ class ProviderTemplateTest(HeatTestCase):
             'Parameters': {
                 'Foo': {'Type': 'String'},
                 'AList': {'Type': 'CommaDelimitedList'},
+                'MemList': {'Type': 'CommaDelimitedList'},
                 'ListEmpty': {'Type': 'CommaDelimitedList'},
                 'ANum': {'Type': 'Number'},
                 'AMap': {'Type': 'Json'},
@@ -115,6 +117,7 @@ class ProviderTemplateTest(HeatTestCase):
             properties_schema = {
                 "Foo": {"Type": "String"},
                 "AList": {"Type": "List"},
+                "MemList": {"Type": "List"},
                 "ListEmpty": {"Type": "List"},
                 "ANum": {"Type": "Number"},
                 "AMap": {"Type": "Map"}
@@ -125,8 +128,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         map_prop_val = {
@@ -140,6 +143,8 @@ class ProviderTemplateTest(HeatTestCase):
         prop_vals = {
             "Foo": "Bar",
             "AList": ["one", "two", "three"],
+            "MemList": [{"key": "name", "value": "three"},
+                        {"key": "name", "value": "four"}],
             "ListEmpty": [],
             "ANum": 5,
             "AMap": map_prop_val,
@@ -158,6 +163,12 @@ class ProviderTemplateTest(HeatTestCase):
         self.assertEqual("Bar", converted_params.get("Foo"))
         # verify List conversion
         self.assertEqual("one,two,three", converted_params.get("AList"))
+        # verify Member List conversion
+        mem_exp = ('.member.0.key=name,'
+                   '.member.0.value=three,'
+                   '.member.1.key=name,'
+                   '.member.1.value=four')
+        self.assertEqual(mem_exp, converted_params.get("MemList"))
         # verify Number conversion
         self.assertEqual(5, converted_params.get("ANum"))
         # verify Map conversion
@@ -198,8 +209,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -227,8 +238,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -260,8 +271,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -292,8 +303,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -322,8 +333,8 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'DummyResource': 'test_resource.template'}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template, files=files),
-                             env=env,
+                             parser.Template(empty_template, files=files,
+                             env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -356,7 +367,7 @@ class ProviderTemplateTest(HeatTestCase):
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(
                                  {'HeatTemplateFormatVersion': '2012-12-12'},
-                                 files=files), env=env,
+                                 files=files, env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -392,7 +403,7 @@ class ProviderTemplateTest(HeatTestCase):
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(
                                  {'HeatTemplateFormatVersion': '2012-12-12'},
-                                 files=files), env=env,
+                                 files=files, env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -424,7 +435,7 @@ class ProviderTemplateTest(HeatTestCase):
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(
                                  {'HeatTemplateFormatVersion': '2012-12-12'},
-                                 files=files), env=env,
+                                 files=files, env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -434,14 +445,120 @@ class ProviderTemplateTest(HeatTestCase):
                                                       definition, stack)
         self.assertIsNone(temp_res.validate())
 
-    def test_get_template_resource(self):
-        # assertion: if the name matches {.yaml|.template} we get the
-        # TemplateResource class.
+    def test_resource_info_general(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Foo': {'Type': 'Boolean'},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider),
+                 'foo.template': json.dumps(provider)}
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.BOOLEAN)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template',
+                   'resources': {'foo': 'foo.template'}}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files, env=env),
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": "False"})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        self.assertEqual('test_resource.template',
+                         temp_res.template_name)
+
+    def test_resource_info_special(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Foo': {'Type': 'Boolean'},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider),
+                 'foo.template': json.dumps(provider)}
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.BOOLEAN)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template',
+                   'resources': {'foo': {'DummyResource': 'foo.template'}}}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files, env=env),
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('foo',
+                                                  'DummyResource',
+                                                  {"Foo": "False"})
+        temp_res = template_resource.TemplateResource('foo',
+                                                      definition, stack)
+        self.assertEqual('foo.template',
+                         temp_res.template_name)
+
+    def test_get_error_for_invalid_template_name(self):
+        # assertion: if the name matches {.yaml|.template} and is valid
+        # we get the TemplateResource class, otherwise error will be raised.
         env_str = {'resource_registry': {'resources': {'fred': {
             "OS::ResourceType": "some_magic.yaml"}}}}
         env = environment.Environment(env_str)
-        cls = env.get_class('OS::ResourceType', 'fred')
-        self.assertEqual(template_resource.TemplateResource, cls)
+        ex = self.assertRaises(exception.NotFound, env.get_class,
+                               'OS::ResourceType', 'fred')
+        self.assertIn('Could not fetch remote template "some_magic.yaml"',
+                      six.text_type(ex))
+
+    def test_metadata_update_called(self):
+        provider = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'Foo': {'Type': 'Boolean'},
+            },
+        }
+        files = {'test_resource.template': json.dumps(provider)}
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.BOOLEAN)}
+            attributes_schema = {}
+
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(
+                                 {'HeatTemplateFormatVersion': '2012-12-12'},
+                                 files=files, env=env),
+                             stack_id=str(uuid.uuid4()))
+
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  "DummyResource",
+                                                  {"Foo": "False"})
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition, stack)
+        temp_res.metadata_set = mock.Mock()
+        temp_res.metadata_update()
+        temp_res.metadata_set.assert_called_once_with({})
 
     def test_get_template_resource_class(self):
         test_templ_name = 'file:///etc/heatr/frodo.yaml'
@@ -482,8 +599,9 @@ class ProviderTemplateTest(HeatTestCase):
         self.assertTrue(test_templ, "Empty test template")
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
-                     allowed_schemes=('file',))\
-            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
+                     allowed_schemes=('file',)
+                     ).AndRaise(urlfetch.URLFetchError(
+                         _('Failed to retrieve template')))
         urlfetch.get(test_templ_name,
                      allowed_schemes=('http', 'https')).AndReturn(test_templ)
         parsed_test_templ = template_format.parse(test_templ)
@@ -529,7 +647,7 @@ class ProviderTemplateTest(HeatTestCase):
         """
         env = {'resource_registry': {'http://example.com/test.template': None,
                                      'resources': {}}}
-        #A KeyError will be thrown prior to this fix.
+        # A KeyError will be thrown prior to this fix.
         environment.Environment(env=env)
 
     def test_system_template_retrieve_by_file(self):
@@ -569,7 +687,7 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template), env=env,
+                             parser.Template(empty_template, env=env),
                              stack_id=str(uuid.uuid4()))
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -597,8 +715,9 @@ class ProviderTemplateTest(HeatTestCase):
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
                      allowed_schemes=('http', 'https',
-                                      'file'))\
-            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
+                                      'file')
+                     ).AndRaise(urlfetch.URLFetchError(
+                         _('Failed to retrieve template')))
         self.m.ReplayAll()
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -620,13 +739,14 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template), env=env,
+                             parser.Template(empty_template, env=env),
                              stack_id=str(uuid.uuid4()))
 
         self.m.StubOutWithMock(urlfetch, "get")
         urlfetch.get(test_templ_name,
-                     allowed_schemes=('http', 'https'))\
-            .AndRaise(urlfetch.URLFetchError(_('Failed to retrieve template')))
+                     allowed_schemes=('http', 'https')
+                     ).AndRaise(urlfetch.URLFetchError(
+                         _('Failed to retrieve template')))
         self.m.ReplayAll()
 
         definition = rsrc_defn.ResourceDefinition('test_t_res',
@@ -648,7 +768,7 @@ class ProviderTemplateTest(HeatTestCase):
         env.load({'resource_registry':
                   {'Test::Flippy': test_templ_name}})
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                             parser.Template(empty_template), env=env,
+                             parser.Template(empty_template, env=env),
                              stack_id=str(uuid.uuid4()))
 
         self.m.ReplayAll()
@@ -661,316 +781,152 @@ class ProviderTemplateTest(HeatTestCase):
         self.assertRaises(exception.StackValidationFailed, temp_res.validate)
         self.m.VerifyAll()
 
-
-class NestedProvider(HeatTestCase):
-    """Prove that we can use the registry in a nested provider."""
-
-    def setUp(self):
-        super(NestedProvider, self).setUp()
-        utils.setup_dummy_db()
-
-    def test_nested_env(self):
-        main_templ = '''
-heat_template_version: 2013-05-23
-resources:
-  secret2:
-    type: My::NestedSecret
-outputs:
-  secret1:
-    value: { get_attr: [secret1, value] }
-'''
-
-        nested_templ = '''
-heat_template_version: 2013-05-23
-resources:
-  secret2:
-    type: My::Secret
-outputs:
-  value:
-    value: { get_attr: [secret2, value] }
-'''
-
-        env_templ = '''
-resource_registry:
-  "My::Secret": "OS::Heat::RandomString"
-  "My::NestedSecret": nested.yaml
-'''
+    def test_incorrect_template_provided_with_url(self):
+        wrong_template = '''
+         <head prefix="og: http://ogp.me/ns# fb: http://ogp.me/ns/fb#
+        '''
 
         env = environment.Environment()
-        env.load(yaml.load(env_templ))
-        templ = parser.Template(template_format.parse(main_templ),
-                                files={'nested.yaml': nested_templ})
-        stack = parser.Stack(utils.dummy_context(),
-                             utils.random_name(),
-                             templ, env=env)
-        stack.store()
-        stack.create()
-        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
+        test_templ_name = 'http://heatr/bad_tmpl.yaml'
+        env.load({'resource_registry':
+                  {'Test::Tmpl': test_templ_name}})
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(empty_template, env=env),
+                             stack_id=str(uuid.uuid4()))
 
-    def test_no_infinite_recursion(self):
-        """Prove that we can override a python resource.
+        self.m.StubOutWithMock(urlfetch, "get")
+        urlfetch.get(test_templ_name,
+                     allowed_schemes=('http', 'https')
+                     ).AndReturn(wrong_template)
+        self.m.ReplayAll()
 
-        And use that resource within the template resource.
-        """
+        definition = rsrc_defn.ResourceDefinition('test_t_res',
+                                                  'Test::Tmpl')
+        temp_res = template_resource.TemplateResource('test_t_res',
+                                                      definition,
+                                                      stack)
+        err = self.assertRaises(exception.StackValidationFailed,
+                                temp_res.validate)
+        self.assertIn('Error parsing template: ', six.text_type(err))
 
-        main_templ = '''
-heat_template_version: 2013-05-23
-resources:
-  secret2:
-    type: OS::Heat::RandomString
-outputs:
-  secret1:
-    value: { get_attr: [secret1, value] }
-'''
+        self.m.VerifyAll()
 
-        nested_templ = '''
-heat_template_version: 2013-05-23
-resources:
-  secret2:
-    type: OS::Heat::RandomString
-outputs:
-  value:
-    value: { get_attr: [secret2, value] }
-'''
 
-        env_templ = '''
-resource_registry:
-  "OS::Heat::RandomString": nested.yaml
-'''
+class TemplateDataTest(common.HeatTestCase):
+
+    def setUp(self):
+        super(TemplateDataTest, self).setUp()
+        files = {}
+        self.ctx = utils.dummy_context()
+
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.STRING,
+                                                   required=True)}
+            attributes_schema = {}
 
         env = environment.Environment()
-        env.load(yaml.load(env_templ))
-        templ = parser.Template(template_format.parse(main_templ),
-                                files={'nested.yaml': nested_templ})
-        stack = parser.Stack(utils.dummy_context(),
-                             utils.random_name(),
-                             templ, env=env)
-        stack.store()
-        stack.create()
-        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+
+        self.stack = parser.Stack(self.ctx, 'test_stack',
+                                  parser.Template(empty_template, files=files,
+                                                  env=env),
+                                  stack_id=str(uuid.uuid4()))
+
+        self.defn = rsrc_defn.ResourceDefinition('test_t_res',
+                                                 "DummyResource",
+                                                 {"Foo": "bar"})
+        self.res = template_resource.TemplateResource('test_t_res',
+                                                      self.defn, self.stack)
+
+    def test_template_data_in_update_without_template_file(self):
+        self.res.action = self.res.UPDATE
+        self.res.nested = mock.MagicMock()
+        self.res.get_template_file = mock.Mock(
+            side_effect=exception.NotFound())
+        self.assertRaises(exception.NotFound, self.res.template_data)
+
+    def test_template_data_in_create_without_template_file(self):
+        self.res.action = self.res.CREATE
+        self.res.nested = mock.MagicMock()
+        self.res.get_template_file = mock.Mock(
+            side_effect=exception.NotFound())
+        self.assertEqual('{}', self.res.template_data())
 
 
-class ProviderTemplateUpdateTest(HeatTestCase):
-    main_template = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  the_nested:
-    Type: the.yaml
-    Properties:
-      one: my_name
-
-Outputs:
-  identifier:
-    Value: {Ref: the_nested}
-  value:
-    Value: {'Fn::GetAtt': [the_nested, the_str]}
-'''
-
-    main_template_2 = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  the_nested:
-    Type: the.yaml
-    Properties:
-      one: updated_name
-
-Outputs:
-  identifier:
-    Value: {Ref: the_nested}
-  value:
-    Value: {'Fn::GetAtt': [the_nested, the_str]}
-'''
-
-    initial_tmpl = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Parameters:
-  one:
-    Default: foo
-    Type: String
-Resources:
-  NestedResource:
-    Type: OS::Heat::RandomString
-    Properties:
-      salt: {Ref: one}
-Outputs:
-  the_str:
-    Value: {'Fn::GetAtt': [NestedResource, value]}
-'''
-    prop_change_tmpl = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Parameters:
-  one:
-    Default: yikes
-    Type: String
-  two:
-    Default: foo
-    Type: String
-Resources:
-  NestedResource:
-    Type: OS::Heat::RandomString
-    Properties:
-      salt: {Ref: one}
-Outputs:
-  the_str:
-    Value: {'Fn::GetAtt': [NestedResource, value]}
-'''
-    attr_change_tmpl = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Parameters:
-  one:
-    Default: foo
-    Type: String
-Resources:
-  NestedResource:
-    Type: OS::Heat::RandomString
-    Properties:
-      salt: {Ref: one}
-Outputs:
-  the_str:
-    Value: {'Fn::GetAtt': [NestedResource, value]}
-  something_else:
-    Value: just_a_string
-'''
-    content_change_tmpl = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Parameters:
-  one:
-    Default: foo
-    Type: String
-Resources:
-  NestedResource:
-    Type: OS::Heat::RandomString
-    Properties:
-      salt: yum
-Outputs:
-  the_str:
-    Value: {'Fn::GetAtt': [NestedResource, value]}
-'''
-
-    EXPECTED = (UPDATE, NOCHANGE) = ('update', 'nochange')
-    scenarios = [
-        ('no_changes', dict(template=main_template,
-                            provider=initial_tmpl,
-                            expect=NOCHANGE)),
-        ('main_tmpl_change', dict(template=main_template_2,
-                                  provider=initial_tmpl,
-                                  expect=UPDATE)),
-        ('provider_change', dict(template=main_template,
-                                 provider=content_change_tmpl,
-                                 expect=UPDATE)),
-        ('provider_props_change', dict(template=main_template,
-                                       provider=prop_change_tmpl,
-                                       expect=NOCHANGE)),
-        ('provider_attr_change', dict(template=main_template,
-                                      provider=attr_change_tmpl,
-                                      expect=NOCHANGE)),
-    ]
+class TemplateResourceCrudTest(common.HeatTestCase):
+    provider = {
+        'HeatTemplateFormatVersion': '2012-12-12',
+        'Parameters': {
+            'Foo': {'Type': 'String'},
+            'Blarg': {'Type': 'String', 'Default': 'wibble'},
+        },
+    }
 
     def setUp(self):
-        super(ProviderTemplateUpdateTest, self).setUp()
-        self.ctx = utils.dummy_context('test_username', 'aaaa', 'password')
+        super(TemplateResourceCrudTest, self).setUp()
+        files = {'test_resource.template': json.dumps(self.provider)}
+        self.ctx = utils.dummy_context()
 
-    def create_stack(self):
-        t = template_format.parse(self.main_template)
-        tmpl = parser.Template(t, files={'the.yaml': self.initial_tmpl})
-        stack = parser.Stack(self.ctx, utils.random_name(), tmpl)
-        stack.store()
-        stack.create()
-        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
-        return stack
+        class DummyResource(object):
+            support_status = support.SupportStatus()
+            properties_schema = {"Foo":
+                                 properties.Schema(properties.Schema.STRING,
+                                                   required=True)}
+            attributes_schema = {}
 
-    def test_template_resource_update_template_schema(self):
-        stack = self.create_stack()
-        self.stack = stack
-        initial_id = stack.output('identifier')
-        initial_val = stack.output('value')
-        tmpl = parser.Template(template_format.parse(self.template),
-                               files={'the.yaml': self.provider})
-        updated_stack = parser.Stack(self.ctx, stack.name, tmpl)
-        stack.update(updated_stack)
-        self.assertEqual(('UPDATE', 'COMPLETE'), stack.state)
-        self.assertEqual(initial_id,
-                         stack.output('identifier'))
-        if self.expect == self.NOCHANGE:
-            self.assertEqual(initial_val,
-                             stack.output('value'))
-        else:
-            self.assertNotEqual(initial_val,
-                                stack.output('value'))
-        self.m.VerifyAll()
+        env = environment.Environment()
+        resource._register_class('DummyResource', DummyResource)
+        env.load({'resource_registry':
+                  {'DummyResource': 'test_resource.template'}})
+        self.stack = parser.Stack(self.ctx, 'test_stack',
+                                  parser.Template(empty_template, files=files,
+                                                  env=env),
+                                  stack_id=str(uuid.uuid4()))
 
+        self.defn = rsrc_defn.ResourceDefinition('test_t_res',
+                                                 "DummyResource",
+                                                 {"Foo": "bar"})
+        self.res = template_resource.TemplateResource('test_t_res',
+                                                      self.defn, self.stack)
+        self.assertIsNone(self.res.validate())
 
-class ProviderTemplateAdoptTest(HeatTestCase):
-    main_template = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Resources:
-  the_nested:
-    Type: the.yaml
-    Properties:
-      one: my_name
+    def test_handle_create(self):
+        self.res.create_with_template = mock.Mock(return_value=None)
 
-Outputs:
-  identifier:
-    Value: {Ref: the_nested}
-  value:
-    Value: {'Fn::GetAtt': [the_nested, the_str]}
-'''
+        self.res.handle_create()
 
-    nested_tmpl = '''
-HeatTemplateFormatVersion: '2012-12-12'
-Parameters:
-  one:
-    Default: foo
-    Type: String
-Resources:
-  RealRandom:
-    Type: OS::Heat::RandomString
-    Properties:
-      salt: {Ref: one}
-Outputs:
-  the_str:
-    Value: {'Fn::GetAtt': [RealRandom, value]}
-'''
+        self.res.create_with_template.assert_called_once_with(
+            self.provider, {'Foo': 'bar'})
 
-    def setUp(self):
-        super(ProviderTemplateAdoptTest, self).setUp()
-        self.ctx = utils.dummy_context('test_username', 'aaaa', 'password')
+    def test_handle_adopt(self):
+        self.res.create_with_template = mock.Mock(return_value=None)
 
-    def _yaml_to_json(self, yaml_templ):
-        return yaml.load(yaml_templ)
+        self.res.handle_adopt(resource_data={'resource_id': 'fred'})
 
-    def test_abandon(self):
-        t = template_format.parse(self.main_template)
-        tmpl = parser.Template(t, files={'the.yaml': self.nested_tmpl})
-        stack = parser.Stack(self.ctx, utils.random_name(), tmpl)
-        stack.store()
-        stack.create()
-        self.assertEqual((stack.CREATE, stack.COMPLETE), stack.state)
-        info = stack.prepare_abandon()
-        self.assertEqual(self._yaml_to_json(self.main_template),
-                         info['template'])
-        self.assertEqual(self._yaml_to_json(self.nested_tmpl),
-                         info['resources']['the_nested']['template'])
-        self.m.VerifyAll()
+        self.res.create_with_template.assert_called_once_with(
+            self.provider, {'Foo': 'bar'}, adopt_data={'resource_id': 'fred'})
 
-    def test_adopt(self):
-        data = {'resources': {u'the_nested': {
-            'resources': {u'RealRandom': {
-                'resource_data': {
-                    u'value': u'N8hE5C7ijdGn4RwnuygbAokGHnTq4cFJ'},
-                'resource_id': 'N8hE5C7ijdGn4RwnuygbAokGHnTq4cFJ'}}}}}
+    def test_handle_update(self):
+        self.res.update_with_template = mock.Mock(return_value=None)
 
-        t = template_format.parse(self.main_template)
-        tmpl = parser.Template(t, files={'the.yaml': self.nested_tmpl})
-        stack = parser.Stack(self.ctx, utils.random_name(), tmpl,
-                             adopt_stack_data=data)
-        self.stack = stack
-        stack.store()
-        stack.adopt()
+        self.res.handle_update(self.defn, None, None)
 
-        self.assertEqual(('ADOPT', 'COMPLETE'), stack.state)
-        nested_res = data['resources']['the_nested']['resources']
-        self.assertEqual(nested_res['RealRandom']['resource_data']['value'],
-                         stack.output('value'))
+        self.res.update_with_template.assert_called_once_with(
+            self.provider, {'Foo': 'bar'})
 
-        self.m.VerifyAll()
+    def test_handle_delete(self):
+        self.res.rpc_client = mock.MagicMock()
+        self.res.id = 55
+        self.res.uuid = six.text_type(uuid.uuid4())
+        self.res.resource_id = six.text_type(uuid.uuid4())
+        self.res.action = self.res.CREATE
+        ident = identifier.HeatIdentifier(self.ctx.tenant_id,
+                                          self.res.physical_resource_name(),
+                                          self.res.resource_id)
+        self.res.handle_delete()
+        rpcc = self.res.rpc_client.return_value
+        rpcc.delete_stack.assert_called_once_with(self.ctx, ident)
