@@ -16,24 +16,26 @@ import json
 import uuid
 
 import mock
-from oslo.config import cfg
+from oslo_config import cfg
 import six
 
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common import short_id
 from heat.common import timeutils
-from heat.db import api as db_api
 from heat.engine import attributes
 from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import dependencies
 from heat.engine import environment
 from heat.engine import parser
 from heat.engine import resource
+from heat.engine import resources
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.engine import template
-from heat.tests.common import HeatTestCase
+from heat.objects import resource as resource_objects
+from heat.objects import resource_data as resource_data_object
+from heat.tests import common
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
@@ -43,7 +45,7 @@ import neutronclient.common.exceptions as neutron_exp
 empty_template = {"HeatTemplateFormatVersion": "2012-12-12"}
 
 
-class ResourceTest(HeatTestCase):
+class ResourceTest(common.HeatTestCase):
     def setUp(self):
         super(ResourceTest, self).setUp()
 
@@ -59,16 +61,18 @@ class ResourceTest(HeatTestCase):
                        u'ResourceWithCustomConstraint'}})
 
         self.stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                                  parser.Template(empty_template),
-                                  env=self.env, stack_id=str(uuid.uuid4()))
+                                  parser.Template(empty_template,
+                                                  env=self.env),
+                                  stack_id=str(uuid.uuid4()))
         self.patch('heat.engine.resource.warnings')
 
     def test_get_class_ok(self):
-        cls = resource.get_class('GenericResourceType')
+        cls = resources.global_env().get_class('GenericResourceType')
         self.assertEqual(generic_rsrc.GenericResource, cls)
 
     def test_get_class_noexist(self):
-        self.assertRaises(exception.StackValidationFailed, resource.get_class,
+        self.assertRaises(exception.StackValidationFailed,
+                          resources.global_env().get_class,
                           'NoExistResourceType')
 
     def test_resource_new_ok(self):
@@ -91,8 +95,9 @@ class ResourceTest(HeatTestCase):
         snippet = rsrc_defn.ResourceDefinition('aresource',
                                                'GenericResourceType')
         self.stack.id = None
-        db_method = 'resource_get_by_name_and_stack'
-        with mock.patch.object(db_api, db_method) as resource_get:
+        db_method = 'get_by_name_and_stack'
+        with mock.patch.object(resource_objects.Resource,
+                               db_method) as resource_get:
             res = resource.Resource('aresource', snippet, self.stack)
             self.assertEqual("INIT", res.action)
             self.assertIs(False, resource_get.called)
@@ -118,6 +123,24 @@ class ResourceTest(HeatTestCase):
         self.assertEqual((res.INIT, res.COMPLETE), res.state)
         self.assertEqual('', res.status_reason)
 
+    def test_signal_wrong_action_state(self):
+        snippet = rsrc_defn.ResourceDefinition('res',
+                                               'GenericResourceType')
+        res = resource.Resource('res', snippet, self.stack)
+        actions = [res.SUSPEND, res.DELETE]
+        for action in actions:
+            for status in res.STATUSES:
+                res.state_set(action, status)
+                ev = self.patchobject(res, '_add_event')
+                ex = self.assertRaises(exception.ResourceFailure,
+                                       res.signal)
+                self.assertEqual('Exception: resources.res: '
+                                 'Cannot signal resource during '
+                                 '%s' % action, six.text_type(ex))
+                ev.assert_called_with(
+                    action, status,
+                    'Cannot signal resource during %s' % action)
+
     def test_resource_str_repr_stack_id_resource_id(self):
         tmpl = rsrc_defn.ResourceDefinition('test_res_str_repr', 'Foo')
         res = generic_rsrc.GenericResource('test_res_str_repr', tmpl,
@@ -168,7 +191,7 @@ class ResourceTest(HeatTestCase):
         self.assertIsNotNone(res.id)
         expected = '%s-%s-%s' % (self.stack.name,
                                  res.name,
-                                 short_id.get_id(res.id))
+                                 short_id.get_id(res.uuid))
         self.assertEqual(expected, res.physical_resource_name_or_FnGetRefId())
 
         # otherwise use parent method
@@ -282,6 +305,31 @@ class ResourceTest(HeatTestCase):
         self.assertRaises(
             resource.UpdateReplace, scheduler.TaskRunner(res.update, utmpl))
 
+    def test_update_replace_in_failed_without_nested(self):
+        tmpl = rsrc_defn.ResourceDefinition('test_resource',
+                                            'GenericResourceType',
+                                            {'Foo': 'abc'})
+        res = generic_rsrc.ResourceWithProps('test_resource', tmpl, self.stack)
+        res.update_allowed_properties = ('Foo',)
+        self.m.StubOutWithMock(generic_rsrc.ResourceWithProps, 'handle_create')
+        generic_rsrc.ResourceWithProps.handle_create().AndRaise(
+            exception.ResourceFailure)
+        self.m.ReplayAll()
+
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(res.create))
+        self.assertEqual((res.CREATE, res.FAILED), res.state)
+
+        utmpl = rsrc_defn.ResourceDefinition('test_resource',
+                                             'GenericResourceType',
+                                             {'Foo': 'xyz'})
+        # resource in failed status and hasn't nested will enter
+        # UpdateReplace flow
+        self.assertRaises(
+            resource.UpdateReplace, scheduler.TaskRunner(res.update, utmpl))
+
+        self.m.VerifyAll()
+
     def test_updated_time_changes_only_on_update_calls(self):
         tmpl = rsrc_defn.ResourceDefinition('test_resource',
                                             'GenericResourceType')
@@ -301,7 +349,7 @@ class ResourceTest(HeatTestCase):
         self.assertEqual(res.IN_PROGRESS, res.status)
         self.assertEqual('test_store', res.status_reason)
 
-        db_res = db_api.resource_get(res.context, res.id)
+        db_res = resource_objects.Resource.get_obj(res.context, res.id)
         self.assertEqual(res.CREATE, db_res.action)
         self.assertEqual(res.IN_PROGRESS, db_res.status)
         self.assertEqual('test_store', db_res.status_reason)
@@ -310,6 +358,7 @@ class ResourceTest(HeatTestCase):
         self.assertEqual(res.CREATE, res.action)
         self.assertEqual(res.COMPLETE, res.status)
         self.assertEqual('test_update', res.status_reason)
+        db_res.refresh()
         self.assertEqual(res.CREATE, db_res.action)
         self.assertEqual(res.COMPLETE, db_res.status)
         self.assertEqual('test_update', db_res.status_reason)
@@ -495,7 +544,8 @@ class ResourceTest(HeatTestCase):
         tmpl = rsrc_defn.ResourceDefinition(rname, 'Foo', {})
         res = generic_rsrc.ResourceWithRequiredProps(rname, tmpl, self.stack)
 
-        estr = 'Property error : test_resource: Property Foo not assigned'
+        estr = ('Property error: test_resource.Properties: '
+                'Property Foo not assigned')
         create = scheduler.TaskRunner(res.create)
         err = self.assertRaises(exception.ResourceFailure, create)
         self.assertIn(estr, six.text_type(err))
@@ -507,7 +557,9 @@ class ResourceTest(HeatTestCase):
                                             {'Food': 'abc'})
         res = generic_rsrc.ResourceWithProps(rname, tmpl, self.stack)
 
-        estr = 'StackValidationFailed: Unknown Property Food'
+        estr = ('StackValidationFailed: resources.test_resource: '
+                'Property error: test_resource.Properties: '
+                'Unknown Property Food')
         create = scheduler.TaskRunner(res.create)
         err = self.assertRaises(exception.ResourceFailure, create)
         self.assertIn(estr, six.text_type(err))
@@ -585,7 +637,8 @@ class ResourceTest(HeatTestCase):
                                      status_reason='just because'))
         self.m.ReplayAll()
 
-        estr = ('ResourceInError: Went to status ERROR due to "just because"')
+        estr = ('ResourceInError: resources.test_resource: '
+                'Went to status ERROR due to "just because"')
         create = scheduler.TaskRunner(res.create)
         err = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(estr, six.text_type(err))
@@ -851,6 +904,28 @@ class ResourceTest(HeatTestCase):
         self.assertEqual(res.FAILED, res.status)
         self.assertIn('boom', res.status_reason)
 
+    def test_verify_check_conditions(self):
+        valid_foos = ['foo1', 'foo2']
+        checks = [
+            {'attr': 'foo1', 'expected': 'bar1', 'current': 'baz1'},
+            {'attr': 'foo2', 'expected': valid_foos, 'current': 'foo2'},
+            {'attr': 'foo3', 'expected': 'bar3', 'current': 'baz3'},
+            {'attr': 'foo4', 'expected': 'foo4', 'current': 'foo4'},
+            {'attr': 'foo5', 'expected': valid_foos, 'current': 'baz5'},
+        ]
+        tmpl = rsrc_defn.ResourceDefinition('test_res', 'GenericResourceType')
+        res = generic_rsrc.ResourceWithProps('test_res', tmpl, self.stack)
+
+        exc = self.assertRaises(exception.Error,
+                                res._verify_check_conditions, checks)
+        exc_text = six.text_type(exc)
+        self.assertNotIn("'foo2':", exc_text)
+        self.assertNotIn("'foo4':", exc_text)
+        self.assertIn("'foo1': expected 'bar1', got 'baz1'", exc_text)
+        self.assertIn("'foo3': expected 'bar3', got 'baz3'", exc_text)
+        self.assertIn("'foo5': expected '['foo1', 'foo2']', got 'baz5'",
+                      exc_text)
+
     def test_suspend_resume_ok(self):
         tmpl = rsrc_defn.ResourceDefinition('test_resource',
                                             'GenericResourceType',
@@ -1031,7 +1106,7 @@ class ResourceTest(HeatTestCase):
         self.assertFalse(res.is_using_neutron())
 
     def _test_skip_validation_if_custom_constraint(self, tmpl):
-        stack = parser.Stack(utils.dummy_context(), 'test', tmpl, env=self.env)
+        stack = parser.Stack(utils.dummy_context(), 'test', tmpl)
         stack.store()
         path = ('heat.engine.clients.os.neutron.NetworkConstraint.'
                 'validate_with_client')
@@ -1052,7 +1127,7 @@ class ResourceTest(HeatTestCase):
                     }
                 }
             }
-        })
+        }, env=self.env)
         self._test_skip_validation_if_custom_constraint(tmpl)
 
     def test_hot_ref_skip_validation_if_custom_constraint(self):
@@ -1067,11 +1142,11 @@ class ResourceTest(HeatTestCase):
                     }
                 }
             }
-        })
+        }, env=self.env)
         self._test_skip_validation_if_custom_constraint(tmpl)
 
 
-class ResourceAdoptTest(HeatTestCase):
+class ResourceAdoptTest(common.HeatTestCase):
     def setUp(self):
         super(ResourceAdoptTest, self).setUp()
         resource._register_class('GenericResourceType',
@@ -1130,8 +1205,9 @@ class ResourceAdoptTest(HeatTestCase):
         }
         adopt = scheduler.TaskRunner(res.adopt, res_data)
         adopt()
-        self.assertEqual("test-value",
-                         db_api.resource_data_get(res, "test-key"))
+        self.assertEqual(
+            "test-value",
+            resource_data_object.ResourceData.get_val(res, "test-key"))
         self.assertEqual({"os_distro": "test-distro"}, res.metadata_get())
         self.assertEqual({"os_distro": "test-distro"}, res.metadata)
         self.assertEqual((res.ADOPT, res.COMPLETE), res.state)
@@ -1156,11 +1232,11 @@ class ResourceAdoptTest(HeatTestCase):
         res = self.stack['foo']
         adopt = scheduler.TaskRunner(res.adopt, None)
         self.assertRaises(exception.ResourceFailure, adopt)
-        expected = 'Exception: Resource ID was not provided.'
+        expected = 'Exception: resources.foo: Resource ID was not provided.'
         self.assertEqual(expected, res.status_reason)
 
 
-class ResourceDependenciesTest(HeatTestCase):
+class ResourceDependenciesTest(common.HeatTestCase):
     def setUp(self):
         super(ResourceDependenciesTest, self).setUp()
 
@@ -1358,6 +1434,30 @@ class ResourceDependenciesTest(HeatTestCase):
                                stack.validate)
         self.assertIn('"baz" (in bar.Properties.Foo)', six.text_type(ex))
 
+    def test_validate_value_fail(self):
+        tmpl = template.Template({
+            'heat_template_version': '2013-05-23',
+            'resources': {
+                'bar': {
+                    'type': 'ResourceWithPropsType',
+                    'properties': {
+                        'FooInt': 'notanint',
+                    }
+                }
+            }
+        })
+        stack = parser.Stack(utils.dummy_context(), 'test', tmpl)
+        ex = self.assertRaises(exception.StackValidationFailed,
+                               stack.validate)
+        self.assertIn("Property error: resources.bar.properties.FooInt: "
+                      "Value 'notanint' is not an integer",
+                      six.text_type(ex))
+
+        # You can turn off value validation via strict_validate
+        stack_novalidate = parser.Stack(utils.dummy_context(), 'test', tmpl,
+                                        strict_validate=False)
+        self.assertIsNone(stack_novalidate.validate())
+
     def test_getatt(self):
         tmpl = template.Template({
             'HeatTemplateFormatVersion': '2012-12-12',
@@ -1636,7 +1736,7 @@ class ResourceDependenciesTest(HeatTestCase):
         self.assertIn('"wibble" (in foo)', six.text_type(ex))
 
 
-class MetadataTest(HeatTestCase):
+class MetadataTest(common.HeatTestCase):
     def setUp(self):
         super(MetadataTest, self).setUp()
         self.stack = parser.Stack(utils.dummy_context(),
@@ -1670,7 +1770,7 @@ class MetadataTest(HeatTestCase):
         self.assertEqual(test_data, self.res.metadata)
 
 
-class ReducePhysicalResourceNameTest(HeatTestCase):
+class ReducePhysicalResourceNameTest(common.HeatTestCase):
     scenarios = [
         ('one', dict(
             limit=10,
@@ -1735,3 +1835,92 @@ class ReducePhysicalResourceNameTest(HeatTestCase):
             else:
                 # check that nothing has changed
                 self.assertEqual(self.original, reduced)
+
+
+class ResourceHookTest(common.HeatTestCase):
+
+    def setUp(self):
+        super(ResourceHookTest, self).setUp()
+
+        resource._register_class('GenericResourceType',
+                                 generic_rsrc.GenericResource)
+        resource._register_class('ResourceWithCustomConstraint',
+                                 generic_rsrc.ResourceWithCustomConstraint)
+
+        self.env = environment.Environment()
+        self.env.load({u'resource_registry':
+                      {u'OS::Test::GenericResource': u'GenericResourceType',
+                       u'OS::Test::ResourceWithCustomConstraint':
+                       u'ResourceWithCustomConstraint'}})
+
+        self.stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                                  parser.Template(empty_template,
+                                                  env=self.env),
+                                  stack_id=str(uuid.uuid4()))
+
+    def test_hook(self):
+        snippet = rsrc_defn.ResourceDefinition('res',
+                                               'GenericResourceType')
+        res = resource.Resource('res', snippet, self.stack)
+
+        res.data = mock.Mock(return_value={})
+        self.assertFalse(res.has_hook('pre-create'))
+        self.assertFalse(res.has_hook('pre-update'))
+
+        res.data = mock.Mock(return_value={'pre-create': 'True'})
+        self.assertTrue(res.has_hook('pre-create'))
+        self.assertFalse(res.has_hook('pre-update'))
+
+        res.data = mock.Mock(return_value={'pre-create': 'False'})
+        self.assertFalse(res.has_hook('pre-create'))
+        self.assertFalse(res.has_hook('pre-update'))
+
+        res.data = mock.Mock(return_value={'pre-update': 'True'})
+        self.assertFalse(res.has_hook('pre-create'))
+        self.assertTrue(res.has_hook('pre-update'))
+
+    def test_set_hook(self):
+        snippet = rsrc_defn.ResourceDefinition('res',
+                                               'GenericResourceType')
+        res = resource.Resource('res', snippet, self.stack)
+
+        res.data_set = mock.Mock()
+        res.data_delete = mock.Mock()
+
+        res.trigger_hook('pre-create')
+        res.data_set.assert_called_with('pre-create', 'True')
+
+        res.trigger_hook('pre-update')
+        res.data_set.assert_called_with('pre-update', 'True')
+
+        res.clear_hook('pre-create')
+        res.data_delete.assert_called_with('pre-create')
+
+    def test_signal_clear_hook(self):
+        snippet = rsrc_defn.ResourceDefinition('res',
+                                               'GenericResourceType')
+        res = resource.Resource('res', snippet, self.stack)
+
+        res.clear_hook = mock.Mock()
+        res.has_hook = mock.Mock(return_value=True)
+        self.assertRaises(exception.ResourceActionNotSupported,
+                          res.signal, None)
+        self.assertFalse(res.clear_hook.called)
+
+        self.assertRaises(exception.ResourceActionNotSupported,
+                          res.signal, {})
+        self.assertFalse(res.clear_hook.called)
+
+        self.assertRaises(exception.ResourceActionNotSupported,
+                          res.signal, {'unset_hook': 'unknown_hook'})
+        self.assertFalse(res.clear_hook.called)
+
+        res.signal({'unset_hook': 'pre-create'})
+        res.clear_hook.assert_called_with('pre-create')
+
+        res.signal({'unset_hook': 'pre-update'})
+        res.clear_hook.assert_called_with('pre-update')
+
+        res.has_hook = mock.Mock(return_value=False)
+        self.assertRaises(exception.ResourceActionNotSupported,
+                          res.signal, {'unset_hook': 'pre-create'})
